class info_elasticBeams():

    def __init__(self, data, iRun):

        ##################################################
        ### Static data (same for all batch instances) ###
        ##################################################

        # General project information
        self.author = data[1][0]  # Name of author
        self.company = data[1][1]  # Company name
        self.job_name = data[1][2]  # Project name

        # Batch mode options (False -> results only returned for given iSet)
        self.batchMode = data[1][3]
        # Input set for a single analysis (ignored if batchMode = True)
        self.iSet = data[1][4]
        self.iRun = int(iRun)
        self.tbOption = data[1][5]

        # Generate Unique Names for XTRACT
        # CAUTION: Modify standard values below with care
        # In almost all cases, editing is not required
        self.mat_name_unc = 'AutoUnconfined'  # Unconfinced concrete
        self.mat_name_conc = 'AutoConfined'  # Confined concrete
        self.mat_name_steel = 'AutoSteel'  # Steel
        self.section_name = 'AutoSection'  # Section name
        self.material_core = 'AutoConfined'  # Concrete core material name
        self.material_cover = 'AutoUnconfined'  # Concrete cover material name
        self.material_steel = 'AutoSteel'  # Steel material name

        # Specify path to save
        self.pSave = data[1][6]

        # Specify option to write terminal to log gile
        self.write_log = data[1][7]

        #####################
        ### Instance Data ###
        #####################

        # File name
        self.file_name = data[0].loc['info_filename', iRun]

        # Material data
        self.Ec = data[0].loc['Ec', iRun]  # Modulus of elasticity (MPa)
        self.den = data[0].loc['den', iRun]  # Density of concrete (kg/m3)
        self.v = data[0].loc['v', iRun]  # Poisson's ratio of concrete

        # General section properties
        # 'Rectangular Beam', 'Rectangular Column', 'Circular', 'T Beam'
        self.section_type = data[0].loc['section_type', iRun]  # Section type
        self.cover = data[0].loc['cover', iRun]  # concrete cover (mm)

        # Geometry for circular sections
        # Section Diameter (mm)
        self.diameter_circ = data[0].loc['diameter_circ', iRun]

        # Geometry for all rectangular sections
        self.rect_width = data[0].loc['rect_width', iRun]  # Section width (mm)
        # Section height (mm)
        self.rect_height = data[0].loc['rect_height', iRun]

        # Geometry for T Beam sections
        self.t_width = data[0].loc['t_width', iRun]  # Total section width (mm)
        # Total section height (mm)
        self.t_height = data[0].loc['t_height', iRun]
        self.f_width = data[0].loc['f_width', iRun]  # Flange width (mm)
        self.f_thick = data[0].loc['f_thick', iRun]  # Flange thickness (mm)

        # Properties for Cruciform sections
        self.bX = data[0].loc['bX', iRun]  # Total section width (mm)
        self.bY = data[0].loc['bY', iRun]  # Total section height (mm)
        self.tX = data[0].loc['tX', iRun]  # Vertical flange thickness (mm)
        # Horizontal flange thickness (mm)
        self.tY = data[0].loc['tY', iRun]
        # Eccentricity of vertical flange (mm)
        self.cX = data[0].loc['cX', iRun]
        # Eccentricity of horizontal flange (mm)
        self.cY = data[0].loc['cY', iRun]

        # Section, material and curve nomenclature
        try:
            # Name sections and curves based on Beam ID
            self.beam_num = int(self.file_name[2:])

        except (IndexError, ValueError) as error:
            # Name sections and curves based on run number
            self.beam_num = int(self.iRun + 1) * 10

        # Handle area updating for initialization
        self.section_area = 0.0
        if self.section_type == 'Circular':
            self.section_area = 3.14 * \
                pow(float(self.diameter_circ) / 2000, 2.0)
        elif self.section_type == 'Rectangular Beam':
            self.section_area = float(
                self.rect_width) * float(self.rect_height) / (1000**2)
        elif self.section_type == 'Rectangular Column':
            self.section_area = round((float(self.rect_width) / 1000)
                                      * float(self.rect_height) / 1000, 5)
        elif self.section_type == 'T Beam':
            self.section_area = (float(self.f_width) * float(self.f_thick) +
                                 float(self.t_width) * (float(self.t_height) -
                                                        float(self.f_thick))) / (1000**2)
        elif self.section_type == 'Cruciform':
            # Check inputs
            if float(self.bX) - float(self.tX) <= 0 or float(self.bY) - \
                    float(self.tY) <= 0:
                raise AssertionError('Cruciform flange thickness is greater\
                                     than total section width!')
            if float(self.bX) - float(self.cX) <= 0 or float(self.bY) - \
                    float(self.cY) <= 0:
                raise AssertionError('Cruciform flange eccentricity is too\
                                     large!')
            self.section_area = round(((float(self.tX) *
                                        float(self.bY) +
                                        float(self.tY) *
                                        float(self.bX) -
                                        float(self.tX) *
                                        float(self.tY)) /
                                       (1000**2)), 5)

    def updateArea(self, area):
        # Handle area updating for logic loops
        if self.section_type == 'Circular':
            self.section_area = 3.14 * \
                pow(float(self.diameter_circ) / 2000, 2.0)
        elif self.section_type == 'Rectangular Beam':
            self.section_area = float(
                self.rect_width) * float(self.rect_height) / (1000**2)
        elif self.section_type == 'Rectangular Column':
            self.section_area = round((float(self.rect_width) / 1000)
                                      * float(self.rect_height) / 1000, 5)
        elif self.section_type == 'T Beam':
            self.section_area = (float(self.f_width) * float(self.f_thick) +
                                 float(self.t_width) * (float(self.t_height) -
                                                        float(self.f_thick))) / (1000**2)
        elif self.section_type == 'Cruciform':
            self.section_area = round(((float(self.tX) *
                                        float(self.bY) +
                                        float(self.tY) *
                                        float(self.bX) -
                                        float(self.tX) *
                                        float(self.tY)) /
                                       (1000**2)), 5)

        return self.section_area



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 12:39:44 2018

@author: kevin.stanton

This class uses the autopy function to run an XTRACT file and export the
analysis results. The output file is a ".txt" file that is saved and moved in
the current working directory.

Notes:
    Developed on a 1920x1080 display. All mouse movements are normalized and
    scaled by these values.
"""

import tkinter as tk
import subprocess
from autopy3 import key
import time
import psutil
import os
from .util import util


class Autorun:

    def __init__(self, aData, mcSettings, xpj_name=''):

        # Get current working directory and screen size
        self.cwd = str(aData.pSave)

        # Get screen dimensions
        root = tk.Tk()
        self.sWidth = root.winfo_screenwidth()
        self.sHeight = root.winfo_screenheight()
        self.sWidthF = self.sWidth / 1920
        self.sHeightF = self.sHeight / 1080
        root.quit()

        # Get settings data
        self.mcSettings = mcSettings

        # Get autodyna module path
        import autodyna.Resources.XTRACT as pathIndex
        self.xtractPath = os.path.dirname(pathIndex.__file__)
        
        # Determine path to .xpj file to run
        if xpj_name == '':
            self.xpj_name = self.cwd + r'\\' + str(aData.file_name) + '.xpj'
        else:
            self.xpj_name = os.path.join(os.getcwd(), xpj_name)

    def __launch_app__(self, aData):

        # Launch the application
        try:
            p = psutil.Popen([aData.pXTRACT, self.xpj_name],
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        except FileNotFoundError:
            try:
                p = psutil.Popen(['C:\\Program Files (x86)\\TRC\\XTRACT\\XTRACT.exe',
                                  self.xpj_name],
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            except FileNotFoundError:
                try:
                    p = psutil.Popen(['C:\\ProgramData\\XTRACT\\XTRACT.exe',
                                      self.xpj_name],
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
                except FileNotFoundError:
                    try:
                        p = psutil.Popen([os.path.join(self.xtractPath,
                                                       'XTRACT.exe'),
                                          self.xpj_name],
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE)
                    except FileNotFoundError:
                        raise AssertionError(
                            'Error: XTRACT not properly installed/associated with wrong path')

        # Assign subproccess
        self.proc = p
        util.confirm(aData)
        if not aData.mInter:
            time.sleep(self.mcSettings.sleepLaunch)

    def __run_anal__(self, aData, n_strainlim):
        time.sleep(2)
        util.click(1, 1)
        time.sleep(1)
        util.click(1, 1)
        time.sleep(1)
        key.tap(key.K_F5)  # Run analysis
        print('\n   Running XTRACT...')
        util.confirm(aData)

        if aData.timingControl == 'automatic':
            # CPU usage tracking
            time.sleep(15)
            cpu_hist, stop_flag = util.sample_cpu_percent(self.proc)
            time.sleep(2)
        else:
            # Fixed analysis timing
            c = 1  # Reduce run time if plots are not shown
            if aData.showPlots:
                c = 0.9
            analysis_time = c * (self.mcSettings.sleepAt1 * 2 + float(n_strainlim) * self.mcSettings.sleepAt2) * \
                float(aData.section_area * self.mcSettings.sleepAt3 * 1e6) / float(600 * 600)
            if not aData.mInter:
                print('\n            sleeping for ' +
                      str(round(analysis_time +
                                self.mcSettings.sleepRunAnal, 1)) +
                      ' seconds \n')
                time.sleep(analysis_time)
                time.sleep(self.mcSettings.sleepRunAnal)

    def __show_res__(self, aData, n_strainlim):

        util.click(1, 1)  # Upper-left corner to get to window
        time.sleep(1)
        util.click(1, 1)
        time.sleep(1)
        key.toggle('d', True, key.MOD_CONTROL)
        key.toggle('d', False, key.MOD_CONTROL)
        time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2)
        util.click(int(self.mcSettings.clickPosition1 *
                       self.sWidthF), int(self.mcSettings.clickPosition2 *
                                          self.sHeightF), 'right') # Click 'Project Name'
        time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2)
        util.click(int(self.mcSettings.clickPosition3 *
                       self.sWidthF), int(self.mcSettings.clickPosition4 *
                                          self.sHeightF)) # Click 'Show Data'
        util.confirm(aData)
        if not aData.mInter:
            time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2)

    def __select_data__(self, aData, n_strainlim):

        # Add cwd to clipboard
        saveName = os.path.join(self.cwd,str(aData.file_name) + '.txt')
        util.addToClipBoard(saveName)

        # Select and save data
        util.click(int(self.mcSettings.clickPosition5 *
                       self.sWidthF), int(self.mcSettings.clickPosition6 *
                                          self.sHeightF)) # Click 'Select All'
        time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2)
        util.click(int(self.mcSettings.clickPosition7 *
                       self.sWidthF), int(self.mcSettings.clickPosition6 *
                                          self.sHeightF)) # Click 'Save Data'
        time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2 * 0.5)
        key.toggle('v', True, key.MOD_CONTROL)  # Paste cwd string
        key.toggle('v', False, key.MOD_CONTROL)
        time.sleep(self.mcSettings.sleepPaste)
        key.tap(key.K_RIGHT) # Move to end of pasted string
        if self.mcSettings.controlMode == 'default':
            key.tap(key.K_BACKSPACE) # Clear trailing space
        key.tap(key.K_RETURN) # Execute cd command
        time.sleep(self.mcSettings.sleepSLc1 + self.mcSettings.sleepSLc2)
        util.click(int(self.mcSettings.clickPosition8 *
                       self.sWidthF), int(self.mcSettings.clickPosition9 *
                                          self.sHeightF)) # Select dialog box at outermost level
        key.tap(key.K_RETURN)  # Save
        time.sleep(self.mcSettings.sleepSave)

    def __close_app__(self):

        # Close the program
        os.system("TASKKILL /F /IM xtract.exe")
        time.sleep(1)

    def autorun_xtract(self, aData, n_strainlim=1):

        # Autorun XTRACT analysis
        self.__launch_app__(aData)
        self.__run_anal__(aData, n_strainlim)
        self.__show_res__(aData, n_strainlim)
        self.__select_data__(aData, n_strainlim)
        saveName = os.path.join(self.cwd,str(aData.file_name) + '.txt')

        # Check for errors and correct if possible
        if os.path.isfile(saveName) == False:
            time.sleep(10)
            if os.path.isfile(saveName):
                self.__show_res__(aData, n_strainlim)
                self.__select_data__(aData, n_strainlim)
            else:
                time.sleep(20)
                if os.path.isfile(saveName):
                    self.__show_res__(aData, n_strainlim)
                    self.__select_data__(aData, n_strainlim)
                else:
                    time.sleep(20)
                    self.__show_res__(aData, n_strainlim)
                    self.__select_data__(aData, n_strainlim)

        self.__close_app__()



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 12:54:15 2018

@author: kevin.stanton

This class initializes an XTRACT model given user defined data ported from
AutoMHB.py
"""

import os
import datetime
import time
import math


class iXTRACT_envelope():

    def __init__(self, aData):

        # Delete xtract and txt file if they exist
        if os.path.exists(str(aData.pSave) + '\\' +
                          str(aData.file_name) + '.txt'):
            os.remove(str(aData.pSave) + '\\' + str(aData.file_name) + '.txt')
        if os.path.exists(str(aData.pSave) + '\\' +
                          str(aData.file_name) + '.xpj'):
            os.remove(str(aData.pSave) + '\\' + str(aData.file_name) + '.xpj')

        # Open a file handle
        self.fwrite = open(str(aData.pSave) + '\\' +
                           str(aData.file_name) + '.xpj', mode='w')

    def __intro__(self, aData):

        # Write intro statements
        self.fwrite.write('\n')
        self.fwrite.write('# Project file generated by Python\n')
        self.fwrite.write('# Created by: %s\n' % aData.author)
        self.fwrite.write('\n')
        self.fwrite.write(
            '# WARNING: When this file is read into XTRACT, data consistency\n')
        self.fwrite.write(
            '# checks are NOT performed. If inappropriate input data is read\n')
        self.fwrite.write(
            '# into the software, unexpected results may ensue. Before modifying\n')
        self.fwrite.write(
            '# this file, it is advised to save a back up copy.\n')
        self.fwrite.write(
            '# The order of the file in not important, nor is the spacing.\n')
        self.fwrite.write(
            '# The file must contain Global data as described below.\n')
        self.fwrite.write(
            '# Note that if this file is saved outside XTRACT, the associated\n')
        self.fwrite.write(
            '# output file will be automatically deleted when the file is opened.\n')
        self.fwrite.write('\n')

    def __global_params__(self, aData,
                          version='3.08',  # Xtract version
                          units='N-m'):  # Analysis units

        # Define datetime stamp
        stamp = datetime.datetime.fromtimestamp(time.time()).\
            strftime('%m/%d/%Y %I:%M:%S %p')

        # Write gloabl parameters
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')
        self.fwrite.write('\n')
        self.fwrite.write('# Define global parameters for the file\n')
        self.fwrite.write('# Version, Units and Name are required.\n')
        self.fwrite.write('Begin_Global\n')
        self.fwrite.write('	NAME = %s\n' % aData.file_name)
        self.fwrite.write('	VER = %s\n' % version)
        self.fwrite.write('	UNITS = %s\n' % units)
        self.fwrite.write('\n')
        self.fwrite.write('	# Optional parameters\n')
        self.fwrite.write('	Author = %s\n' % aData.author)
        self.fwrite.write('	Company = %s\n' % aData.company)
        self.fwrite.write('	Job_Name = %s\n' % aData.job_name)
        self.fwrite.write('	Time_Stamp = %s\n' % stamp)
        self.fwrite.write('End_Global\n')
        self.fwrite.write('\n')
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')
        self.fwrite.write('\n')

    def __mater_conu__(self, aData):

        # Write material definitions for unconfined concrete
        self.fwrite.write('# Begin unconfined concrete material definition\n')
        self.fwrite.write('Begin_Material\n')
        self.fwrite.write('	NAME = %s\n' % aData.mat_name_unc)
        self.fwrite.write('	TYPE = Unconfined Concrete\n')
        self.fwrite.write('	Fc = %s\n' % str(round(float(aData.Fc) * 1e6, 1)))
        self.fwrite.write('	Ft = %s\n' % str(round(float(aData.Ft) * 1e6, 1)))
        self.fwrite.write('	ey = %s\n' % str(round(float(aData.ey), 5)))
        self.fwrite.write('	ecu = %s\n' % str(round(float(aData.ecu), 5)))
        self.fwrite.write('	esp = %s\n' % str(round(float(aData.esp), 5)))
        self.fwrite.write('	ef = %s\n' % str(round(float(aData.ef), 5)))
        self.fwrite.write('	Ec = %s\n' % str(round(float(aData.Ec) * 1e6, 1)))
        self.fwrite.write('End_Material\n')
        self.fwrite.write('\n')
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')
        self.fwrite.write('\n')

    def __mater_conc__(self, aData):

        # Write material definitions for confined concrete
        self.fwrite.write('# Begin confined concrete material definition\n')
        self.fwrite.write('Begin_Material\n')
        self.fwrite.write('	NAME = %s\n' % aData.mat_name_conc)
        self.fwrite.write('	TYPE = Confined Concrete\n')
        self.fwrite.write('	Fc = %s\n' % str(round(float(aData.Fc) * 1e6, 1)))
        self.fwrite.write('	Ft = %s\n' % str(round(float(aData.Ft) * 1e6, 1)))
        self.fwrite.write('	ey = %s\n' % str(round(float(aData.ey), 5)))
        self.fwrite.write('	ecu = %s\n' % str(round(float(aData.ecu), 5)))
        self.fwrite.write('	Fl = %s\n' % str(round(float(aData.Fc) * 1e6, 1)))
        self.fwrite.write('	Ec = %s\n' % str(round(float(aData.Ec) * 1e6, 1)))
        self.fwrite.write('\n')
        self.fwrite.write(
            '	# Optional parameters for confinement calculators\n')
        self.fwrite.write('	compStrength = %s\n' %
                          str(round(float(aData.Fc) * 1e6, 1)))
        self.fwrite.write('End_Material\n')
        self.fwrite.write('\n')
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')
        self.fwrite.write('\n')

    def __mater_ste__(self, aData):

        # Write material definitions for steel
        self.fwrite.write('# Begin steel material definition\n')
        self.fwrite.write('Begin_Material\n')
        self.fwrite.write('	NAME = %s\n' % aData.mat_name_steel)
        self.fwrite.write('	TYPE = Strain Hardening Steel\n')
        self.fwrite.write('	Fy = %s\n' % str(round(float(aData.Fy) * 1e6, 1)))
        self.fwrite.write('	Fu = %s\n' % str(round(float(aData.Fu) * 1e6, 1)))
        self.fwrite.write('	esh = %s\n' % str(round(float(aData.esh), 5)))
        self.fwrite.write('	esu = %s\n' % str(round(float(aData.esu), 5)))
        self.fwrite.write('	Es = %s\n' % str(round(float(aData.Es) * 1e6, 1)))
        self.fwrite.write('End_Material\n')
        self.fwrite.write('\n')
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')
        self.fwrite.write('\n')

    def __sect__(self, aData, concrete_strainlim, steel_strainlim):

        # Begin section definition
        self.fwrite.write('# Begin section definitions.\n')
        self.fwrite.write('Begin_Section\n')
        envelope_name = aData.section_name + '-C' + \
            str(concrete_strainlim) + '-S' + str(steel_strainlim * aData.esy)

        # Write section builder properties for circular section
        if aData.section_type == 'Circular':
            section_area = 3.14 * pow(float(aData.diameter_circ) / 2000, 2.0)
            mesh = str(round(float(aData.diameter_circ) / 1000 * 0.05, 5))
            min_triangle_area = str(round(pow(float(mesh), 2) / 3.0, 5))
            max_n_fiber = round(1000 * section_area /
                                pow(float(mesh) * 1000, 2) * 2000)

            self.fwrite.write('	Begin_Builder\n')
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('  Boundary_Bottom = -%s\n' %
                              str(round(float(aData.diameter_circ) / 1000 * 2, 5)))
            self.fwrite.write('  Boundary_Left = -%s\n' %
                              str(round(float(aData.diameter_circ) / 1000 * 2, 5)))
            self.fwrite.write('  Boundary_Right = %s\n' %
                              str(round(float(aData.diameter_circ) / 1000 * 2, 5)))
            self.fwrite.write('  Boundary_Top = %s\n' %
                              str(round(float(aData.diameter_circ) / 1000 * 2, 5)))
            self.fwrite.write('  Min_Triangle_Area = %s\n' %
                              min_triangle_area)
            self.fwrite.write('  Max_Number_of_Fibers = %s\n' %
                              str(max_n_fiber))
            self.fwrite.write(
                '  Window_Left = %s\n' % str(
                    round(
                        float(
                            aData.diameter_circ) / 1000 * 2.5,
                        5)))
            self.fwrite.write('  Window_Bottom = %s\n' % str(
                round(float(aData.diameter_circ) / 1000 * 1.25, 5)))
            self.fwrite.write('  Window_Height = %s\n' % str(
                round(float(aData.diameter_circ) / 1000 * 2.5, 5)))
            self.fwrite.write(' End_Builder\n')
            self.fwrite.write('\n')

        # Write section builder properties for rectangular beam section
        elif aData.section_type == 'Rectangular Beam':
            section_area = float(aData.rect_width) / 1000 * \
                float(aData.rect_height) / 1000
            # mesh length in m
            mesh = str(min(round(
                ((float(aData.rect_height) + float(aData.rect_width)) / 2000) * 0.05, 5), 0.028))
            min_triangle_area = str(round(pow(float(mesh), 2) / 3.0, 5))
            max_n_fiber = round(1000 * section_area /
                                pow(float(mesh) * 1000, 2) * 2000)

            self.fwrite.write('	Begin_Builder\n')
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('		NAME = %s\n' % envelope_name)
            self.fwrite.write('		Boundary_Bottom = -%s\n' %
                              str(round(float(aData.rect_height) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write('		Boundary_Left = -%s\n' %
                              str(round(float(aData.rect_width) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write('		Boundary_Right = %s\n' % str(
                round(float(aData.rect_width) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write(
                '		Boundary_Top = %s\n' % str(
                    round(
                        float(
                            aData.rect_height) / 1000 / 2 + 50 / 1000,
                        5)))
            self.fwrite.write('		Min_Triangle_Area = %s\n' %
                              min_triangle_area)
            self.fwrite.write('		Max_Number_of_Fibers = %s\n'
                              % str(max_n_fiber))
            self.fwrite.write(
                '		Window_Left = %s\n' % str(
                    round(
                        float(
                            aData.rect_width) / 1000 * 2 - 100 / 1000,
                        5)))
            self.fwrite.write(
                '		Window_Bottom = %s\n' % str(
                    round(
                        float(
                            aData.rect_height) / 1000 / 2 + 100 / 1000,
                        5)))
            self.fwrite.write('		Window_Height = %s\n' % str(
                round(float(aData.rect_height) / 1000 + 200 / 1000, 5)))
            self.fwrite.write('	End_Builder\n')
            self.fwrite.write('\n')

        # Write section builder properties for rectangular column section
        elif aData.section_type == 'Rectangular Column':
            # mesh length in m
            mesh = str(min(round(
                ((float(aData.rect_height) + float(aData.rect_width)) / 2000) * 0.05, 5), 0.028))
            section_area = round((float(aData.rect_width) / 1000)
                                 * float(aData.rect_height) / 1000, 5)
            # triangle size in m^2
            min_triangle_area = str(round(pow(float(mesh), 2) / 3.0, 5))
            max_n_fiber = round(1000 * section_area /
                                pow(float(mesh) * 1000, 2) * 2000)

            self.fwrite.write('	Begin_Builder\n')
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('		NAME = %s\n' % envelope_name)
            self.fwrite.write('		Boundary_Bottom = -%s\n' %
                              str(round(float(aData.rect_height) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write('		Boundary_Left = -%s\n' %
                              str(round(float(aData.rect_width) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write('		Boundary_Right = %s\n' % str(
                round(float(aData.rect_width) / 1000 / 2 + 50 / 1000, 5)))
            self.fwrite.write(
                '		Boundary_Top = %s\n' % str(
                    round(
                        float(
                            aData.rect_height) / 1000 / 2 + 50 / 1000,
                        5)))
            self.fwrite.write('		Min_Triangle_Area = %s\n' %
                              min_triangle_area)
            self.fwrite.write('		Max_Number_of_Fibers = %s\n'
                              % str(max_n_fiber))
            self.fwrite.write(
                '		Window_Left = %s\n' % str(
                    round(
                        float(
                            aData.rect_width) / 1000 * 2 - 0.1,
                        5)))
            self.fwrite.write('		Window_Bottom = %s\n' % str(
                round(float(aData.rect_height) / 1000 / 2 + 0.1, 5)))
            self.fwrite.write('		Window_Height = %s\n' %
                              str(round(float(aData.rect_height) / 1000 + 0.1, 5)))
            self.fwrite.write('	End_Builder\n')
            self.fwrite.write('\n')

        # Write section builder properties for T Beam section
        elif aData.section_type == 'T Beam':
            section_area = (float(aData.f_width) / 1000) * float(aData.f_thick) / 1000 + float(
                aData.t_width) / 1000 * (float(aData.t_height) / 1000 - float(aData.f_thick) / 1000)
            mesh = str(round(min(float(aData.t_height) / 1000,
                                 float(aData.f_width) / 1000) * 0.05, 5))
            max_n_fiber = round(1000 * section_area /
                                pow(float(mesh) * 1000, 2) * 2000) * 5
            # triangle size in m^2
            min_triangle_area = str(pow(float(mesh), 2) / 3.0)

            self.fwrite.write('	Begin_Builder\n')
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('		NAME = %s\n' % envelope_name)
            self.fwrite.write('		Boundary_Bottom = -%s\n' %
                              str(float(aData.t_height) / 1000 + 100 / 1000))
            self.fwrite.write('		Boundary_Left = -%s\n' %
                              str(float(aData.f_width) / 1000 + 100 / 1000))
            self.fwrite.write('		Boundary_Right = %s\n' %
                              str(float(aData.f_width) / 1000 + 100 / 1000))
            self.fwrite.write('		Boundary_Top = %s\n' %
                              str(float(aData.t_height) / 1000 + 100 / 1000))
            self.fwrite.write('		Min_Triangle_Area = %s\n' %
                              min_triangle_area)
            self.fwrite.write('		Max_Number_of_Fibers = %s\n'
                              % str(max_n_fiber))
            self.fwrite.write(
                '		Window_Left = %s\n' % str(
                    round(
                        float(
                            aData.t_width) / 1000 + 200 / 1000,
                        5)))
            self.fwrite.write(
                '		Window_Bottom = %s\n' % str(
                    round(
                        float(
                            aData.t_height) / 1000 / 2 + 100 / 1000,
                        5)))
            self.fwrite.write('		Window_Height = %s\n' % str(
                round(float(aData.t_height) / 1000 + 200 / 1000, 5)))
            self.fwrite.write('	End_Builder\n')
            self.fwrite.write('\n')

        # Write section builder properties for Cruciform section
        elif aData.section_type == 'Cruciform':
            section_area = round(((float(aData.tX) *
                                   float(aData.bY) +
                                   float(aData.tY) *
                                   float(aData.bX) -
                                   float(aData.tX) *
                                   float(aData.tY)) /
                                  (1000**2)), 5)
            mesh = str(round(min(float(aData.tX) / 1000,
                                 float(aData.tY) / 1000) * 0.05, 5))
            max_n_fiber = round(1000 * section_area /
                                pow(float(mesh) * 1000, 2) * 2000) * 5
            # Min triangle size in m^2
            min_triangle_area = str(round(pow(float(mesh), 2) / 3.0, 5))

            self.fwrite.write('	Begin_Builder\n')
            self.fwrite.write('  NAME = %s\n' % envelope_name)
            self.fwrite.write('		NAME = %s\n' % envelope_name)
            self.fwrite.write('		Boundary_Bottom = -%s\n' %
                              str(float(aData.bY) / 1000 + 100 / 1000))
            self.fwrite.write('		Boundary_Left = -%s\n' %
                              str(round(float(aData.bX) / 1000 + 100 / 1000, 2)))
            self.fwrite.write('		Boundary_Right = %s\n' %
                              str(round(float(aData.bX) / 1000 + 100 / 1000, 2)))
            self.fwrite.write('		Boundary_Top = %s\n' %
                              str(round(float(aData.bY) / 1000 + 100 / 1000, 2)))
            self.fwrite.write('		Min_Triangle_Area = %s\n' %
                              min_triangle_area)
            self.fwrite.write('		Max_Number_of_Fibers = %s\n'
                              % str(max_n_fiber))
            self.fwrite.write('		Window_Left = %s\n' %
                              str(round(float(aData.bX) / 1000 + 200 / 1000, 5)))
            self.fwrite.write('		Window_Bottom = %s\n' % str(
                round(float(aData.bY) / 1000 / 2 + 100 / 1000, 5)))
            self.fwrite.write('		Window_Height = %s\n' %
                              str(round(float(aData.bY) / 1000 + 200 / 1000, 5)))
            self.fwrite.write('	End_Builder\n')
            self.fwrite.write('\n')

        # Write section shape for circular section
        if aData.section_type == 'Circular':
            radius = str(float(aData.diameter_circ) / 2000)
            mesh = str(round(float(aData.diameter_circ) / 1000 * 0.05, 5))
            self.fwrite.write('	Begin_Shape\n')
            self.fwrite.write('		MATERIAL = %s\n' % aData.material_cover)
            self.fwrite.write('		MESH = %s\n' % mesh)
            self.fwrite.write('		COVER = %s\n' %
                              str(round(float(aData.cover) / 1000, 5)))
            self.fwrite.write('		MATERIAL_CORE = %s\n' % aData.material_core)
            self.fwrite.write('		Begin_Arc\n')
            self.fwrite.write('			0, -%s\n' % radius)
            self.fwrite.write('			%s, 0\n' % radius)
            self.fwrite.write('			0, %s\n' % radius)
            self.fwrite.write('			-%s, 0\n' % radius)
            self.fwrite.write('			0, -%s\n' % radius)
            self.fwrite.write('		End_Arc\n')
            self.fwrite.write('	End_Shape\n')
            self.fwrite.write('\n')

        # Write section shape for rectangular section
        elif aData.section_type == 'Rectangular Beam':
            # mesh length in m
            mesh = str(
                round(
                    ((float(
                        aData.rect_height) +
                        float(
                        aData.rect_width)) /
                        2000) *
                    0.05,
                    5))
            self.fwrite.write('	Begin_Shape\n')
            self.fwrite.write('		MATERIAL = %s\n' % aData.material_cover)
            self.fwrite.write('		MESH = %s\n' % mesh)
            self.fwrite.write('		COVER = %s\n' %
                              str(round(float(aData.cover) / 1000, 5)))
            self.fwrite.write('		MATERIAL_CORE = %s\n' % aData.material_core)
            self.fwrite.write('		Begin_Line\n')
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, %s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('		End_Line\n')
            self.fwrite.write('	End_Shape\n')
            self.fwrite.write('\n')

        # Write section shape for rectangular column section
        elif aData.section_type == 'Rectangular Column':
            # mesh length in m
            mesh = str(
                round(
                    ((float(
                        aData.rect_height) +
                        float(
                        aData.rect_width)) /
                        2000) *
                    0.05,
                    5))
            self.fwrite.write('	Begin_Shape\n')
            self.fwrite.write('		MATERIAL = %s\n' % aData.material_cover)
            self.fwrite.write('		MESH = %s\n' % mesh)
            self.fwrite.write('		COVER = %s\n' %
                              str(round(float(aData.cover) / 1000, 5)))
            self.fwrite.write('		MATERIAL_CORE = %s\n' % aData.material_core)
            self.fwrite.write('		Begin_Line\n')
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, %s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.rect_width) / 1000 / 2, 5)),
                               str(round(float(aData.rect_height) / 1000 / 2, 5))))
            self.fwrite.write('		End_Line\n')
            self.fwrite.write('	End_Shape\n')
            self.fwrite.write('\n')

        # Write section shape for T Beam section
        elif aData.section_type == 'T Beam':
            mesh = str(round(min(float(aData.t_height) / 1000,
                                 float(aData.f_width) / 1000) * 0.05, 5))
            self.fwrite.write('	Begin_Shape\n')
            self.fwrite.write('		MATERIAL = %s\n' % aData.material_cover)
            self.fwrite.write('		MESH = %s\n' % mesh)
            self.fwrite.write('		COVER = %s\n' %
                              str(round(float(aData.cover) / 1000, 5)))
            self.fwrite.write('		MATERIAL_CORE = %s\n' % aData.material_core)
            self.fwrite.write('		Begin_Line\n')
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.t_width) / 1000 / 2, 5)),
                               str(round(float(aData.t_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, -%s\n' %
                              (str(round(float(aData.t_width) / 1000 / 2, 5)),
                               str(round(float(aData.t_height) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.t_width) /
                                         1000 /
                                         2, 5)), str(round(float(aData.t_height) /
                                                           1000 /
                                                           2 -
                                                           float(aData.f_thick) /
                                                           1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.f_width) /
                                         1000 /
                                         2, 5)), str(round(float(aData.t_height) /
                                                           1000 /
                                                           2 -
                                                           float(aData.f_thick) /
                                                           1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.f_width) / 1000 / 2, 5)),
                               str(round(float(aData.t_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, %s\n' %
                              (str(round(float(aData.f_width) / 1000 / 2, 5)),
                               str(round(float(aData.t_height) / 1000 / 2, 5))))
            self.fwrite.write('			-%s, %s\n' %
                              (str(round(float(aData.f_width) /
                                         1000 /
                                         2, 5)), str(round(float(aData.t_height) /
                                                           1000 /
                                                           2 -
                                                           float(aData.f_thick) /
                                                           1000, 5))))
            self.fwrite.write('			-%s, %s\n' %
                              (str(round(float(aData.t_width) /
                                         1000 /
                                         2, 5)), str(round(float(aData.t_height) /
                                                           1000 /
                                                           2 -
                                                           float(aData.f_thick) /
                                                           1000, 5))))
            self.fwrite.write('			-%s, -%s\n' %
                              (str(round(float(aData.t_width) / 1000 / 2, 5)),
                               str(round(float(aData.t_height) / 1000 / 2, 5))))
            self.fwrite.write('		End_Line\n')
            self.fwrite.write('	End_Shape\n')
            self.fwrite.write('\n')

        # Write section shape for Cruciform section
        elif aData.section_type == 'Cruciform':
            mesh = str(round(min(float(aData.tY) / 1000,
                                 float(aData.tX) / 1000) * 0.05, 5))
            self.fwrite.write('	Begin_Shape\n')
            self.fwrite.write('		MATERIAL = %s\n' % aData.material_cover)
            self.fwrite.write('		MESH = %s\n' % mesh)
            self.fwrite.write('		COVER = %s\n' %
                              str(round(float(aData.cover) / 1000, 5)))
            self.fwrite.write('		MATERIAL_CORE = %s\n' % aData.material_core)
            self.fwrite.write('		Begin_Line\n')
            self.fwrite.write('			%s, %s\n' %
                              (str(round(-1 * (float(aData.bX) / 2 + float(aData.cX)) / 1000, 5)),
                               str(round(float(aData.tY) * (-1) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) * (-1) / 1000 / 2, 5)),
                               str(round(float(aData.tY) * (-1) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) * (-1) / 1000 / 2, 5)),
                               str(round(-1 * (float(aData.bY) / 2 + float(aData.cY)) / 1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) / 1000 / 2, 5)),
                               str(round(-1 * (float(aData.bY) / 2 + float(aData.cY)) / 1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) / 1000 / 2, 5)),
                               str(round(float(aData.tY) * (-1) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round((float(aData.bX) / 2 - float(aData.cX)) / 1000, 5)),
                               str(round(float(aData.tY) * (-1) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round((float(aData.bX) /
                                          2 -
                                          float(aData.cX)) /
                                         1000, 5)), str(round(float(aData.tY) /
                                                              1000 /
                                                              2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) / 1000 / 2, 5)),
                               str(round(float(aData.tY) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) /
                                         1000 /
                                         2, 5)), str(round((float(aData.bY) /
                                                            2 -
                                                            float(aData.cY)) /
                                                           1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) * (-1) / 1000 / 2, 5)),
                               str(round((float(aData.bY) / 2 - float(aData.cY)) / 1000, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(float(aData.tX) * (-1) / 1000 / 2, 5)),
                               str(round(float(aData.tY) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(-1 * (float(aData.bX) / 2 + float(aData.cX)) / 1000, 5)),
                               str(round(float(aData.tY) / 1000 / 2, 5))))
            self.fwrite.write('			%s, %s\n' %
                              (str(round(-1 * (float(aData.bX) / 2 + float(aData.cX)) / 1000, 5)),
                               str(round(float(aData.tY) * (-1) / 1000 / 2, 5))))
            self.fwrite.write('		End_Line\n')
            self.fwrite.write('	End_Shape\n')
            self.fwrite.write('\n')

        return section_area

    def __bars_circ__(self, aData):

        # Calculate bar coordinates for circular section
        radius = float(aData.diameter_circ) / 2000
        area_bar = str(
            round(math.pi * (float(aData.bar_diam_circ) / 2000)**2, 5))
        rein_rad = radius - float(aData.cover) / 1000 - float(
            aData.d_shear) / 1000 - float(aData.bar_diam_circ) / 2000
        angles = [i * 2 * math.pi / float(aData.num_bars_circ)
                  for i in range(int(aData.num_bars_circ))]
        rein_x = [str(round(rein_rad * math.cos(angles[i]), 5))
                  for i in range(int(aData.num_bars_circ))]
        rein_y = [str(round(rein_rad * math.sin(angles[i]), 5))
                  for i in range(int(aData.num_bars_circ))]

        # Round to remove excess digits
        rein_x = [str(round(float(i), 5)) for i in rein_x]
        rein_y = [str(round(float(i), 5)) for i in rein_y]

        # Return bar coordinates and area
        return (rein_x, rein_y), area_bar

    def __bars_rect_beam__(self, aData):

        # Calculate bar coordinates for rectangular section
        area_bar = [str(round(math.pi * (float(aData.db_t) / 1000)**2 / 4, 5))] *\
            int(aData.n_t)
        area_bar.extend([str(round(math.pi * (float(aData.db_b) / 1000)**2 / 4, 5))] *
                        int(aData.n_b))
        rein_xlt = [-(float(aData.rect_width) / 1000 / 2 - float(aData.cover) /
                      1000 - float(aData.db_t) / 1000 / 2 - 6 / 1000)]
        rein_xmt = [rein_xlt[0] + (-2 * rein_xlt[0]) / (int(aData.n_t) - 1) * i
                    for i in range(1, int(aData.n_t) - 1)]
        rein_xrt = [
            float(
                aData.rect_width) /
            1000 /
            2 -
            float(
                aData.cover) /
            1000 -
            float(
                aData.db_t) /
            1000 /
            2 -
            6 /
            1000]
        rein_xt = rein_xlt + rein_xmt + rein_xrt
        rein_xlb = [-(float(aData.rect_width) / 1000 / 2 - float(aData.cover) /
                      1000 - float(aData.db_b) / 1000 / 2 - 6 / 1000)]
        rein_xmb = [rein_xlb[0] + (-2 * rein_xlb[0]) / (int(aData.n_b) - 1) * i
                    for i in range(1, int(aData.n_b) - 1)]
        rein_xrb = [
            float(
                aData.rect_width) /
            1000 /
            2 -
            float(
                aData.cover) /
            1000 -
            float(
                aData.db_b) /
            1000 /
            2 -
            6 /
            1000]
        rein_xb = rein_xlb + rein_xmb + rein_xrb
        rein_x = rein_xt + rein_xb
        rein_yt = [float(aData.rect_height) / 1000 / 2 - float(aData.cover) / 1000 -
                   float(aData.db_t) / 1000 / 2 - 6 / 1000] * int(aData.n_t)
        rein_yb = [-(float(aData.rect_height) / 1000 / 2 - float(aData.cover) /
                     1000 - float(aData.db_b) / 1000 / 2 - 6 / 1000)] * int(aData.n_b)
        rein_y = rein_yt + rein_yb
        rein_y = [str(i) for i in rein_y]

        # Round to remove excess digits
        rein_x = [str(round(float(i), 5)) for i in rein_x]
        rein_y = [str(round(float(i), 5)) for i in rein_y]

        # Return bar coordinates and area
        return (rein_x, rein_y), area_bar

    def __bars_rect_col__(self, aData):

        # Check that rebar and cover dimensions are reasonable
        if float(aData.cover) / 1000 + float(aData.bar_diam_col) / \
                1000 >= min(float(aData.rect_height) / 1000, float(aData.rect_width) / 1000):
            raise AssertionError(
                "Rebar diameter too large for given section geometry.")

        # Calculate bar coordinates for rectangular section
        area_bar = [str(round(math.pi * (float(aData.bar_diam_col) / 2000)**2, 5))] *\
            int(aData.num_bars_col)
        rein_xl = -(float(aData.rect_width) / 1000 / 2 - float(aData.cover) / 1000 -
                    float(aData.bar_diam_col) / 1000 / 2)
        rein_xr = float(aData.rect_width) / 1000 / 2 - float(aData.cover) / \
            1000 - float(aData.bar_diam_col) / 1000 / 2

        if str(aData.num_bars_col) == '8':
            rein_xm = rein_xl + (rein_xr - rein_xl) / 2
            rein_x = [rein_xl, rein_xm, rein_xr, rein_xl,
                      rein_xr, rein_xl, rein_xm, rein_xr]
        elif str(aData.num_bars_col) == '4':
            rein_x = [rein_xl, rein_xr, rein_xl, rein_xr]
        elif str(aData.num_bars_col) == '6':
            rein_x = [rein_xl, rein_xl, rein_xl,
                      rein_xr, rein_xr, rein_xr]
        elif str(aData.num_bars_col) == '10':
            xSpace = (rein_xr - rein_xl) / 4
            rein_x = [rein_xl, rein_xl + xSpace, 0, rein_xr - xSpace, rein_xr,
                      rein_xl, rein_xl + xSpace, 0, rein_xr - xSpace, rein_xr]
        elif str(aData.num_bars_col) == '12':
            xSpace = (rein_xr - rein_xl) / 3
            rein_x = [rein_xl, rein_xl + xSpace, rein_xr - xSpace, rein_xr,
                      rein_xl, rein_xr, rein_xl, rein_xr,
                      rein_xl, rein_xl + xSpace, rein_xr - xSpace, rein_xr]
        elif str(aData.num_bars_col) == '14':
            xSpace = (rein_xr - rein_xl) / 3
            rein_x = [rein_xl, rein_xl + xSpace, rein_xr - xSpace, rein_xr,
                      rein_xl, rein_xr, rein_xl, rein_xr, rein_xl, rein_xr,
                      rein_xl, rein_xl + xSpace, rein_xr - xSpace, rein_xr]

        rein_yt = float(aData.rect_height) / 1000 / 2 - float(aData.cover) / 1000 -\
            float(aData.bar_diam_col) / 1000 / 2
        rein_yb = -(float(aData.rect_height) / 1000 / 2 -
                    float(aData.cover) / 1000 - float(aData.bar_diam_col) / 1000 / 2)

        if str(aData.num_bars_col) == '8':
            rein_ym = rein_yb + (rein_yt - rein_yb) / 2
            rein_y = [rein_yt, rein_yt, rein_yt, rein_ym,
                      rein_ym, rein_yb, rein_yb, rein_yb]
        elif str(aData.num_bars_col) == '4':
            rein_y = [rein_yt, rein_yt, rein_yb, rein_yb]
        elif str(aData.num_bars_col) == '6':
            # middle reinforcement y-axis position for 6 bars
            rein_ym = 0.0
            rein_y = [rein_yt, rein_ym, rein_yb,
                      rein_yt, rein_ym, rein_yb]
        elif str(aData.num_bars_col) == '10':
            rein_y = [rein_yt, rein_yt, rein_yt, rein_yt, rein_yt,
                      rein_yb, rein_yb, rein_yb, rein_yb, rein_yb]
        elif str(aData.num_bars_col) == '12':
            ySpace = (rein_yt - rein_yb) / 3
            rein_y = [rein_yt, rein_yt, rein_yt, rein_yt,
                      rein_yt - ySpace, rein_yt - ySpace,
                      rein_yb + ySpace, rein_yb + ySpace,
                      rein_yb, rein_yb, rein_yb, rein_yb]
        elif str(aData.num_bars_col) == '14':
            rein_ym = 0.0
            ySpace = (rein_yt - rein_yb) / 4
            rein_y = [rein_yt, rein_yt, rein_yt, rein_yt,
                      rein_yt - ySpace, rein_yt - ySpace,
                      rein_ym, rein_ym,
                      rein_yb + ySpace, rein_yb + ySpace,
                      rein_yb, rein_yb, rein_yb, rein_yb]

        # Round to remove excess digis
        rein_x = [str(round(float(i), 5)) for i in rein_x]
        rein_y = [str(round(float(i), 5)) for i in rein_y]

        # Return bar coordinates and area
        return (rein_x, rein_y), area_bar

    def __bars_t_beam__(self, aData):

        # Calculate bar coordinates for T beam section
        area_bar = [str(round(math.pi * (float(aData.t_tbar) / 1000)**2 / 4, 5))] *\
            int(aData.t_nt) + [str(round(math.pi *
                                         (float(aData.t_bbar) / 1000)**2 / 4, 5))] * int(aData.t_nb) +\
            [str(round(math.pi * (float(aData.f_bar) / 1000)**2 / 4, 5))] *\
            int(int((float(aData.f_width) / 2000 - float(aData.t_width) / 2000) /
                    float(aData.f_spac) / 1000) * (float(aData.f_lay) / 1000) * 2)

        # Calculate reinforcement layout for the web
        rein_xtl = -(float(aData.t_width) / 1000 / 2 - float(aData.cover) /
                     1000 - float(aData.t_tbar) / 1000 / 2 - 6 / 1000)
        rein_xtr = float(aData.t_width) / 1000 / 2 - float(aData.cover) / \
            1000 - float(aData.t_tbar) / 1000 / 2 - 6 / 1000
        rein_xtm = [rein_xtl + (-2 * rein_xtl) / (int(aData.t_nt) - 1) * i
                    for i in range(1, int(aData.t_nt) - 1)]
        rein_xt = [rein_xtl] + rein_xtm + [rein_xtr]
        rein_xbl = -(float(aData.t_width) / 1000 / 2 - float(aData.cover) /
                     1000 - float(aData.t_bbar) / 1000 / 2 - 6 / 1000)
        rein_xbr = float(aData.t_width) / 1000 / 2 - float(aData.cover) / \
            1000 - float(aData.t_bbar) / 1000 / 2 - 6 / 1000
        rein_xbm = [rein_xbl + (-2 * rein_xbl) / (int(aData.t_nb) - 1) * i
                    for i in range(1, int(aData.t_nb) - 1)]
        rein_xb = [rein_xbl] + rein_xbm + [rein_xbr]
        rein_xw = rein_xt + rein_xb
        rein_yt = [float(aData.t_height) / 1000 / 2 - float(aData.cover) / 1000 -
                   float(aData.t_tbar) / 1000 / 2 - 6 / 1000] * int(aData.t_nt)
        rein_yb = [-(float(aData.t_height) / 1000 / 2 - float(aData.cover) / 1000 -
                     float(aData.t_bbar) / 1000 / 2 - 6 / 1000)] * int(aData.t_nb)
        rein_yw = rein_yt + rein_yb

        # Calculate reinforcement layout for the flanges
        n_barf = int(round((float(aData.f_width) /
                            1000 /
                            2 -
                            float(aData.t_width) /
                            1000 /
                            2) /
                           (float(aData.f_spac) /
                            1000), 0))
        if n_barf == 1:
            rein_flx = [(-float(aData.f_width) / 1000 / 2 -
                         float(aData.t_width) / 1000 / 2) / 2]
            rein_frx = [(float(aData.f_width) / 1000 / 2 +
                         float(aData.t_width) / 1000 / 2) / 2]
        else:
            s_barf = (float(aData.f_width) / 1000 / 2 -
                      float(aData.t_width) / 1000 / 2) / (n_barf)
            rein_flx = [-float(aData.f_width) / 1000 / 2 + s_barf / 2]
            rein_frx = [float(aData.f_width) / 1000 / 2 - s_barf / 2]
            for i in range(n_barf - 1):
                rein_flx.append(round(-float(aData.f_width) / 1000 / 2 +
                                      s_barf / 2 + s_barf * (i + 1), 1))
                rein_frx.append(round(float(aData.f_width) / 1000 / 2 -
                                      s_barf / 2 - s_barf * (i + 1), 1))
        if int(aData.f_lay) == 1:
            rein_fyt = []  # [float(aData.t_height) - float(aData.f_thick)]
            rein_fyb = [float(aData.t_height) / 1000 / 2 -
                        float(aData.f_thick) / 1000 / 2] * n_barf
            rein_fx = rein_flx + rein_frx
        elif int(aData.f_lay) == 2:
            rein_fyt = [float(aData.t_height) / 1000 / 2 - float(aData.cover) /
                        1000 - float(aData.f_bar) / 1000 / 2 - 6 / 1000] * n_barf
            rein_fyb = [float(aData.t_height) / 1000 / 2 - float(aData.f_thick) / 1000 + float(
                aData.cover) / 1000 + float(aData.f_bar) / 1000 / 2 + 6 / 1000] * n_barf
            rein_fx = rein_flx + rein_frx + rein_flx + rein_frx
        else:
            raise AssertionError("Error: Please select either 1 or 2 layers.")

        rein_fy = rein_fyt + rein_fyt + rein_fyb + rein_fyb

        # Combine reinforcement coordinate data
        rein_x = rein_xw + rein_fx
        rein_y = rein_yw + rein_fy

        # Round to remove excess digits
        rein_x = [str(round(float(i), 5)) for i in rein_x]
        rein_y = [str(round(float(i), 5)) for i in rein_y]

        # Return bar coordinates and area
        return (rein_x, rein_y), area_bar

    def __bars_cruciform__(self, aData):

        # Check that rebar and cover dimensions are reasonable
        if float(aData.cover) / 1000 + float(aData.cBar_rows[1]) * max(float(aData.cBar_vFlangeD),
                                                                       float(aData.cBar_centerD)) / 1000 >= float(aData.tX) / 1000 or \
                float(aData.cover) / 1000 + float(aData.cBar_rows[0]) *\
                max(float(aData.cBar_hFlangeD), float(aData.cBar_centerD)) / 1000\
                >= float(aData.tY) / 1000:
            raise AssertionError(
                "Rebar diameter too large for given section geometry!")
        if int(aData.cBar_rows[1]) <= 0 or int(aData.cBar_rows[0]) <= 0:
            raise AssertionError(
                "Number of rebar rows cannot be less than 2 in either direction for cruciform sections!")
        if min(int(aData.cBar_vFlangeN), int(aData.cBar_hFlangeN)) < 4 or int(
                aData.cBar_vFlangeN) % 2 != 0 or int(aData.cBar_hFlangeN) % 2 != 0:
            raise AssertionError(
                "Total of rebars in each flange of a cruciform must be an even number greater than 4!")

        ### Calculate bar areas, spacings, associated coordinates ###
        # Areas
        aBar_vFlange = [str(round(math.pi * (float(aData.cBar_vFlangeD) / 2000)**2, 5))] *\
            int(aData.cBar_vFlangeN)
        aBar_hFlange = [str(round(math.pi * (float(aData.cBar_hFlangeD) / 2000)**2, 5))] *\
            int(aData.cBar_hFlangeN)
        nBar_center = int(aData.cBar_rows[0]) * int(aData.cBar_rows[1])
        aBar_center = [
            str(round(math.pi * (float(aData.cBar_centerD) / 2000)**2, 5))] * nBar_center

        # Spacings
        spacingH_vFlange = ((float(aData.tX) - 2 * float(aData.cover) -
                             float(aData.cBar_vFlangeD)) / 1000) / (int(aData.cBar_rows[1]) - 1)
        spacingV_hFlange = ((float(aData.tY) - 2 * float(aData.cover) -
                             float(aData.cBar_hFlangeD)) / 1000) / (int(aData.cBar_rows[0]) - 1)
        nRows_vFlange = int(float(aData.cBar_vFlangeN)) / \
            (int(aData.cBar_rows[1]))
        nRows_hFlange = int(float(aData.cBar_hFlangeN)) / \
            (int(aData.cBar_rows[0]))
        spacingV_vFlange = (float(aData.bY) - float(aData.tY) -
                            2 * float(aData.cover)) / (1000 * nRows_vFlange)
        spacingH_hFlange = (float(aData.bX) - float(aData.tX) -
                            2 * float(aData.cover)) / (1000 * nRows_hFlange)

        # Coordinates
        rein_vFlange_x = [round(-(float(aData.tX) / 2 - float(aData.cover) - float(aData.cBar_vFlangeD) / 2) /
                                1000 + i * spacingH_vFlange, 5) for i in range(0, int(aData.cBar_rows[1]))]
        rein_vFlange_y = [
            round(
                ((float(
                    aData.bY) /
                    2 -
                    float(
                    aData.cover) -
                    float(
                    aData.cY) -
                    float(
                    aData.cBar_vFlangeD) /
                    2) /
                    1000) -
                i *
                spacingV_vFlange,
                5) for i in range(
                0,
                int(nRows_vFlange))]
        for i in range(0, len(rein_vFlange_y)):
            if i + 1 > len(rein_vFlange_y) / 2:
                rein_vFlange_y[i] = -1 * rein_vFlange_y[int(
                    i - len(rein_vFlange_y) / 2)] - 2 * float(aData.cY) / 1000

        rein_hFlange_y = [round(-(float(aData.tY) / 2 - float(aData.cover) - float(aData.cBar_hFlangeD) / 2) /
                                1000 + i * spacingV_hFlange, 5) for i in range(0, int(aData.cBar_rows[0]))]
        rein_hFlange_x = [round(-((float(aData.bX) / 2 - float(aData.cover) + float(aData.cX) - float(
            aData.cBar_hFlangeD) / 2) / 1000) + i * spacingH_hFlange, 5) for i in range(0, int(nRows_hFlange))]
        for i in range(0, len(rein_hFlange_x)):
            if i + 1 > len(rein_hFlange_x) / 2:
                rein_hFlange_x[i] = -1 * rein_hFlange_x[int(
                    i - len(rein_hFlange_x) / 2)] - 2 * float(aData.cX) / 1000

        # Combine data
        rein_hFlange_xF = [rein_hFlange_x for i in range(
            0, int(aData.cBar_rows[0]))]
        rein_center_xF = [rein_vFlange_x for i in range(
            0, int(aData.cBar_rows[0]))]
        rein_vFlange_xF = [
            rein_vFlange_x for i in range(0, int(nRows_vFlange))]
        rein_hFlange_yF = [
            rein_hFlange_y for i in range(0, int(nRows_hFlange))]
        rein_center_yF = [rein_hFlange_y for i in range(
            0, int(aData.cBar_rows[1]))]
        rein_vFlange_yF = [rein_vFlange_y for i in range(
            0, int(aData.cBar_rows[1]))]
        rein_x = sorted(sum(rein_hFlange_xF, [])) + \
            sorted(sum(rein_center_xF, [])) + sorted(sum(rein_vFlange_xF, []))
        rein_y = sum(rein_hFlange_yF, []) + \
            sum(rein_center_yF, []) + sum(rein_vFlange_yF, [])

        # Assign areas to each bar
#        area_bar = [aBar_hFlange for i in range(0,int(aData.cBar_hFlangeN))] + [aBar_center for i in range(0,nBar_center)]  + [aBar_vFlange for i in range(0,int(aData.cBar_vFlangeN))]
        area_bar = aBar_hFlange + aBar_center + aBar_vFlange

        # Round to remove excess digits
        rein_x = [str(round(float(i), 5)) for i in rein_x]
        rein_y = [str(round(float(i), 5)) for i in rein_y]

        # Return bar coordinates and area
        return (rein_x, rein_y), area_bar

    def __bars__(self, aData, prestress='0'):

        # Write section bars for circular section
        if aData.section_type == 'Circular':
            self.fwrite.write('	Begin_Rebar\n')
            rein, area_bar = self.__bars_circ__(aData)
            for i in range(int(aData.num_bars_circ)):
                self.fwrite.write('		%s, %s, %s, %s, %s\n' %
                                  (rein[0][i], rein[1][i], area_bar,
                                   prestress, aData.material_steel))
            self.fwrite.write('	End_Rebar\n')
            self.fwrite.write('\n')

        # Write section bars for rectangular beam section
        elif aData.section_type == 'Rectangular Beam':
            self.fwrite.write('	Begin_Rebar\n')
            rein, area_bar = self.__bars_rect_beam__(aData)
            for i in range(int(aData.n_t) + int(aData.n_b)):
                self.fwrite.write('		%s, %s, %s, %s, %s\n' %
                                  (rein[0][i], rein[1][i], area_bar[i],
                                   prestress, aData.material_steel))
            self.fwrite.write('	End_Rebar\n')
            self.fwrite.write('\n')

        # Write section bars for rectangular column section
        elif aData.section_type == 'Rectangular Column':
            self.fwrite.write('	Begin_Rebar\n')
            rein, area_bar = self.__bars_rect_col__(aData)
            for i in range(int(aData.num_bars_col)):
                self.fwrite.write('		%s, %s, %s, %s, %s\n' %
                                  (rein[0][i], rein[1][i], area_bar[i],
                                   prestress, aData.material_steel))
            self.fwrite.write('	End_Rebar\n')
            self.fwrite.write('\n')

        # Write section bars for T beam section
        elif aData.section_type == 'T Beam':
            self.fwrite.write('	Begin_Rebar\n')
            rein, area_bar = self.__bars_t_beam__(aData)
            t_fb = (int((float(aData.f_width) /
                         1000 /
                         2 -
                         float(aData.t_width) /
                         1000 /
                         2) /
                        float(aData.f_spac) /
                        1000) *
                    float(aData.f_lay) /
                    1000 *
                    2)
            for i in range(int(int(aData.t_nt) + int(aData.t_nb) + t_fb)):
                self.fwrite.write('		%s, %s, %s, %s, %s\n' %
                                  (rein[0][i], rein[1][i], area_bar[i],
                                   prestress, aData.material_steel))
            self.fwrite.write('	End_Rebar\n')
            self.fwrite.write('\n')

        # Write section bars for Cruciform section
        elif aData.section_type == 'Cruciform':
            self.fwrite.write('	Begin_Rebar\n')
            rein, area_bar = self.__bars_cruciform__(aData)
            for i in range(0, len(area_bar)):
                self.fwrite.write('		%s, %s, %s, %s, %s\n' %
                                  (rein[0][i], rein[1][i], area_bar[i],
                                   prestress, aData.material_steel))
            self.fwrite.write('	End_Rebar\n')
            self.fwrite.write('\n')

    def __load__(
            self,
            aData,
            concrete_strainlim,
            steel_strainlim,
            laststrain=False,
            mphimethod='BiSection'):

        # Convert axial load from kN to N
        sN_load = str(1000 * float(aData.N_load))

        # Write P-M Interaction load case on the x-axis
        self.fwrite.write('	Begin_Loading\n')
        self.fwrite.write('		NAME = PM_load_X\n')
        self.fwrite.write('		TYPE = PM Interaction\n')
        if not aData.showPlots:
            self.fwrite.write('		Include_Graph = False\n')
            self.fwrite.write('		Include_Animation = False\n')
        self.fwrite.write('		N_PM_Points = 100\n')
        self.fwrite.write('		Full_PM_Diagram = True\n')
        self.fwrite.write('		Generate_Code_PM_Diagram = True\n')
        self.fwrite.write('		Include_PM_Curve_Fit = True\n')
        self.fwrite.write('		Angle_of_Loading = 0\n')
        self.fwrite.write('		Axial0 = 0\n')
        self.fwrite.write('		Axial1 = 0\n')
        self.fwrite.write('		Axial_Cap = 0\n')
        self.fwrite.write('		AxialR0 = 0\n')
        self.fwrite.write('		AxialR1 = 0\n')
        self.fwrite.write('		Moment0 = 0\n')
        self.fwrite.write('		Moment1 = 0\n')
        self.fwrite.write('		Moment2 = 0\n')
        self.fwrite.write('		Angle_of_Loading = 0\n')
        self.fwrite.write('\n')
        self.fwrite.write('		Begin_LimitStrains\n')
        self.fwrite.write('		' + aData.mat_name_unc + ', -' +
                          str(concrete_strainlim) + ', 1\n')  # limiting strain
        self.fwrite.write('		' + aData.mat_name_steel +
                          ', -1, ' + str(aData.esy * steel_strainlim) + '\n')
        self.fwrite.write('		' + aData.mat_name_conc +
                          ', -' + str(concrete_strainlim) + ', 1\n')
        self.fwrite.write('		End_LimitStrains\n')
        self.fwrite.write('\n')
        self.fwrite.write('		Begin_LoadUserComments\n')
        self.fwrite.write('		User Comments \n')
        self.fwrite.write('		End_LoadUserComments\n')
        self.fwrite.write('	End_Loading\n')
        self.fwrite.write('\n')

        # Write P-M Interaction load case on the y-axis
        self.fwrite.write('	Begin_Loading\n')
        self.fwrite.write('		NAME = PM_load_Y\n')
        self.fwrite.write('		TYPE = PM Interaction\n')
        if not aData.showPlots:
            self.fwrite.write('		Include_Graph = False\n')
            self.fwrite.write('		Include_Animation = False\n')
        self.fwrite.write('		N_PM_Points = 100\n')
        self.fwrite.write('		Full_PM_Diagram = True\n')
        self.fwrite.write('		Generate_Code_PM_Diagram = True\n')
        self.fwrite.write('		Include_PM_Curve_Fit = True\n')
        self.fwrite.write('		Angle_of_Loading = 90\n')
        self.fwrite.write('		Axial0 = 0\n')
        self.fwrite.write('		Axial1 = 0\n')
        self.fwrite.write('		Axial_Cap = 0\n')
        self.fwrite.write('		AxialR0 = 0\n')
        self.fwrite.write('		AxialR1 = 0\n')
        self.fwrite.write('		Moment0 = 0\n')
        self.fwrite.write('		Moment1 = 0\n')
        self.fwrite.write('		Moment2 = 0\n')
        self.fwrite.write('		Angle_of_Loading = 0\n')
        self.fwrite.write('\n')
        self.fwrite.write('		Begin_LimitStrains\n')
        self.fwrite.write('		' + aData.mat_name_unc + ', -' +
                          str(concrete_strainlim) + ', 1\n')  # limiting strain
        self.fwrite.write('		' + aData.mat_name_steel +
                          ', -1, ' + str(aData.esy * steel_strainlim) + '\n')
        self.fwrite.write('		' + aData.mat_name_conc +
                          ', -' + str(concrete_strainlim) + ', 1\n')
        self.fwrite.write('		End_LimitStrains\n')
        self.fwrite.write('\n')
        self.fwrite.write('		Begin_LoadUserComments\n')
        self.fwrite.write('		User Comments \n')
        self.fwrite.write('		End_LoadUserComments\n')
        self.fwrite.write('	End_Loading\n')
        self.fwrite.write('\n')

        if laststrain:
            # Write M-φ Interaction load case on the x-axis
            self.fwrite.write('	Begin_Loading\n')
            self.fwrite.write('		NAME = M-phi_Load_X\n')
            self.fwrite.write('		TYPE = Moment Curvature\n')
            if not aData.showPlots:
                self.fwrite.write('		Include_Graph = False\n')
                self.fwrite.write('		Include_Animation = False\n')
            self.fwrite.write('		ConstAxial = -%s\n' % sN_load)
            self.fwrite.write('		IncMxx = 113.0\n')
            self.fwrite.write('		Use_Best_Fit = True\n')
            self.fwrite.write('		Calc_Moment_Rot = False\n')
            self.fwrite.write('		Method = %s\n' % mphimethod)
            self.fwrite.write('		N_Steps_Before_Yield = 10\n')
            self.fwrite.write('		N_Steps_After_Yield = 20\n')
            self.fwrite.write('		Multiple_On_First_Yield = 2\n')
            self.fwrite.write('		BS_Tol = 4448\n')
            self.fwrite.write('		BS_Max_Itter = 40\n')
            self.fwrite.write('	End_Loading\n')

            # Write M-φ Interaction load case on the y-axis
            self.fwrite.write('	Begin_Loading\n')
            self.fwrite.write('		NAME = M-phi_Load_Y\n')
            self.fwrite.write('		TYPE = Moment Curvature\n')
            if not aData.showPlots:
                self.fwrite.write('		Include_Graph = False\n')
                self.fwrite.write('		Include_Animation = False\n')
            self.fwrite.write('		ConstAxial = -%s\n' % sN_load)
            self.fwrite.write('		IncMyy = 113.0\n')
            self.fwrite.write('		Use_Best_Fit = True\n')
            self.fwrite.write('		Calc_Moment_Rot = False\n')
            self.fwrite.write('		Method = %s\n' % mphimethod)
            self.fwrite.write('		N_Steps_Before_Yield = 10\n')
            self.fwrite.write('		N_Steps_After_Yield = 20\n')
            self.fwrite.write('		Multiple_On_First_Yield = 2\n')
            self.fwrite.write('		BS_Tol = 4448\n')
            self.fwrite.write('		BS_Max_Itter = 40\n')
            self.fwrite.write('	End_Loading\n')

        # Close section definition
        self.fwrite.write('End_Section\n')
        self.fwrite.write(
            '#---------------------------------------------------------------------\n')

    def __close__(self):

        # Close file handle
        self.fwrite.close()

    # This will now take a vector of strain limits
    def write_EnvelopeXtract(
            self,
            aData,
            concrete_strainlim,
            steel_strainlim,
            mphimethod='BiSection'):

        # Write input XTRACT file
        self.__intro__(aData)
        self.__global_params__(aData)
        self.__mater_conu__(aData)
        self.__mater_conc__(aData)
        self.__mater_ste__(aData)
        print('   Strain limits considered:')
        instance = 1
        for j in range(0, len(steel_strainlim)):
            for i in range(0, len(concrete_strainlim)):
                print('         ' + str(instance) +
                      ' - Concrete = ' + str(concrete_strainlim[i]))
                print('         ' + str(instance) + ' - Steel = ' +
                      str(float(aData.esy) * steel_strainlim[j]))
                instance = instance + 1
                laststrain = (bool(i == len(concrete_strainlim) - 1)
                              and bool(j == len(steel_strainlim) - 1))
                section_area = self.__sect__(
                    aData, concrete_strainlim[i], steel_strainlim[j])
                self.__bars__(aData)
                self.__load__(
                    aData,
                    concrete_strainlim[i],
                    steel_strainlim[j],
                    laststrain,
                    mphimethod)
        self.__close__()
        return section_area



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 13:01:57 2018

@author: kevin.stanton

This class writes MAT_HYSTERETIC_BEAM card given output from class Shear_Calcs
"""

import math
from decimal import Decimal
from .util import util
import os


class Elastic_Beam():

    def __init__(self, aData):

        # Initialize section area
        self.area = aData.section_area

    def __process__(self, aData):

        # Shape of section
        shape = aData.section_type

        # Define moment of inertia
        if shape == 'Circular':
            # Diameter of circlular section (m)
            D = float(aData.diameter_circ) / 1000.0
            # Uncracked moment of inertia of the section (m4)
            I_xx = math.pi / 4 * (D / 2)**4
            I_yy = math.pi / 4 * (D / 2)**4
            # Cracked moment of inertia of the section (m4)
            I_xx_crack = I_xx / 3
            I_yy_crack = I_yy / 3

        elif (shape == 'Rectangular Beam') |\
             (shape == 'Rectangular Column'):
            # Get the width of the section (m)
            w = float(aData.rect_width) / 1000
            # Get the height of the section (m)
            h = float(aData.rect_height) / 1000
            # Uncracked moment of inertia of the section (m4)
            I_xx = w * h**3 / 12
            I_yy = h * w**3 / 12
            # Cracked moment of inertia of the section (m4)
            I_xx_crack = I_xx / 3
            I_yy_crack = I_yy / 3

        elif shape == 'Cruciform':
            # Get the widths of the section (m)
            bX = float(aData.bX) / 1000
            bY = float(aData.bX) / 1000
            # Get the heights of the section (m)
            hY = float(aData.tY) / 1000
            hX = float(aData.tX) / 1000
            # Uncracked moment of inertia of the section (m4)
            I_xx = bX * hY**3 / 12
            I_yy = bY * hX**3 / 12
            # Cracked moment of inertia of the section (m4)
            I_xx_crack = I_xx / 3
            I_yy_crack = I_yy / 3

        elif shape == 'T Beam':
            # Get the width of the flange (m)
            B = float(aData.f_width) / 1000
            # Get the height of the flange (m)
            h = float(aData.f_thick) / 1000
            # Get the width of the web (m)
            b = float(aData.t_width) / 1000
            # Get the height of the web (m)
            H = float(aData.t_height) / 1000 - float(aData.f_thick) / 1000
            # Get the center of gravity (m)
            A = B * h + H * b
            y_cog = ((H + h / 2) * h * B + H**2 * b / 2) / A
            # Uncracked moment of inertia of the section (m4)
            I_xx = b * H * (y_cog - H / 2)**2 + b * H**3 / 12 + \
                h * B * (H + h / 2 - y_cog)**2 + h**3 * B / 12
            I_yy = b**3 * H / 12 + B**3 * h / 12
            # Cracked moment of inertia of the section (m4)
            I_xx_crack = I_xx / 3
            I_yy_crack = I_yy / 3

        # Define the modulus of elasticity of concrete (Pa)
        E = float(aData.Ec) * 1e6

        # Return output arguments
        return E, I_xx, I_yy, I_xx_crack, I_yy_crack

    def write_mat(self, aData):

        # Initialize curves
        E, I_xx, I_yy, I_xx_crack, I_yy_crack = self.__process__(aData)
        print('\n   Final card data:')
        print('         Young\'s modulus = ' + str(round(E, 0)))
        print('         I_xx = ' + str(round(I_xx, 6)))
        print('         I_xx (cracked) = ' + str(round(I_xx_crack, 6)))
        print('         I_yy = ' + str(round(I_yy, 6)))
        print('         I_yy (cracked) = ' + str(round(I_yy_crack, 6)))
        print('         Section area = ' + str(round(self.area, 5)) + '\n')

        # Input parameters
        zero_10 = '0'.rjust(10)

        # Determine area ratio (<1 if in-slab T-Beam)
        if aData.section_type == 'T-Beam' and aData.tbOption == 'in-slab':
            aEff = aData.section_area - aData.f_width * \
                aData.f_thick  # effective shear area
            f = aEff / aData.section_area  # factor to reduce density written to LS-DYNA
        else:
            f = 1

        # Parameters for first row of material card
        # Define the material id
        mid = str(aData.beam_num).rjust(10)
        # Define mass density of concrete (RO)
        ro = str(f * aData.den).rjust(10)
        # Define Young's modulus of concrete (E)
        e = "{:.1e}".format(E).replace('+', '').rjust(10)
        # Define Poisson's ratio of concrete (PR)
        pr = str(aData.v).rjust(10)

        # Define section card parameters
        # Card 1
        sid = mid  # same as material id
        elform = '2'.rjust(10)
        shrf = '0.0'.rjust(10)
        qririd = '0.0'.rjust(10)
        cst = '0.0'.rjust(10)
        scoor = '0.0'.rjust(10)
        nsm = '0.0'.rjust(10)

        # Card 2
        area = str(float(self.area))
        iss = str(float(I_xx_crack))
        itt = str(float(I_yy_crack))
        area = ("{:.3E}".format(Decimal(area))).rjust(10)
        itt = ("{:.3E}".format(Decimal(itt))).rjust(10)
        iss = ("{:.3E}".format(Decimal(iss))).rjust(10)
        j = '0.0'.rjust(10)
        sa = '0.0'.rjust(10)
        ist = '0.0'.rjust(10)

        # Define material card parameters
        if not aData.pSave:
            keyData = []
            keyData.append('*KEYWORD\n')
            keyData.append('$\n')
            keyData.append('*MAT_ELASTIC_TITLE\n')
            keyData.append(util.getName(aData) + '\n')
            keyData.append(
                '%s%s%s%s%s%s%s%s\n' %
                (mid, ro, e, pr, zero_10, zero_10, zero_10, zero_10))
            keyData.append('$\n')

            # Write section card
            keyData.append('*SECTION_BEAM_TITLE\n')
            keyData.append(util.getName(aData) + '\n')
            keyData.append('%s%s%s%s%s%s%s\n' %
                           (sid, elform, shrf, qririd, cst, scoor, nsm))
            keyData.append('%s%s%s%s%s%s\n' % (area, iss, itt, j, sa, ist))
            keyData.append('$\n')
            keyData.append('*END\n')
        else:
            fwrite = open(
                aData.pSave +
                '\\' +
                util.getName(
                    aData,
                    '.key',
                    check=True),
                'w')
            fwrite.write('*KEYWORD\n')
            fwrite.write('$\n')
            fwrite.write('*MAT_ELASTIC_TITLE\n')
            fwrite.write(util.getName(aData) + '\n')
            fwrite.write('%s%s%s%s%s%s%s%s\n' %
                         (mid, ro, e, pr, zero_10, zero_10, zero_10, zero_10))
            fwrite.write('$\n')

            # Write section card
            fwrite.write('*SECTION_BEAM_TITLE\n')
            fwrite.write(util.getName(aData) + '\n')
            fwrite.write('%s%s%s%s%s%s%s\n' %
                         (sid, elform, shrf, qririd, cst, scoor, nsm))
            fwrite.write('%s%s%s%s%s%s\n' % (area, iss, itt, j, sa, ist))
            fwrite.write('$\n')
            fwrite.write('*END\n')

            # Close file handle
            fwrite.close()

        # Write output in JSON format
        if not aData.pSave == False:
            keyFile = open(
                os.path.join(
                    aData.pSave,
                    util.getName(
                        aData,
                        '.key')),
                'r')
            keyData = keyFile.read()
            keyFile.close()
        jDat = {}
        jDat['file_name'] = util.getName(aData, '.key')
        jDat['value'] = ''.join(keyData)

        return jDat



# -*- coding: utf-8 -*-
"""
Created on Thu Apr 26 13:01:57 2018

@author: kevin.stanton

This class writes MAT_HYSTERETIC_BEAM card given output from class Shear_Calcs
"""

import pandas as pd
import math
import numpy as np
from .Shear_Calcs import Shear_Calcs
from decimal import Decimal
from .biLinearize import biLin
from .util import util
import matplotlib.pyplot as plt
import os


class Mat_Hys_Beam(Shear_Calcs):

    # Envelope is a Boolean input
    def __init__(
            self,
            aData,
            section_area,
            envelope=False,
            n_strainlim=0,
            bilinearflag=False):
        print('         Envelope = ' + str(envelope))

        # Read data in a dataframe format
        # self.P_m must be replaced in order to do the enveloping method
        if not(envelope):
            self.P_m = pd.read_csv(aData.pSave + '\\' + str(aData.file_name) + '.txt',
                                   delim_whitespace=True,
                                   error_bad_lines=False,
                                   skiprows=15).iloc[:, [12, 14, 31, 34]]
            self.M_phi = pd.read_csv(aData.pSave + '\\' + str(aData.file_name) + '.txt',
                                     delim_whitespace=True,
                                     error_bad_lines=False,
                                     skiprows=15).iloc[:, [50, 53, 67, 69]].dropna()
            self.mPhi_long = pd.read_csv(aData.pSave + '\\' + str(aData.file_name) + '.txt',
                                         delim_whitespace=True,
                                         error_bad_lines=False,
                                         skiprows=15).iloc[:, [50, 53, 41, 43, 42, 67, 69, 57, 59, 58]].dropna()
        else:
            # Read from separate CSV
            self.P_m = pd.read_csv(aData.pSave +
                                   '\\' +
                                   str('Moment_Interaction_Envelope' +
                                       aData.file_name +
                                       '.csv'))
            colind = [12, 15, 29, 31]
            colstep = 38
            ind_mphi = [x + colstep * n_strainlim for x in colind]
            self.M_phi = pd.read_csv(aData.pSave + '\\' + str(aData.file_name) + '.txt',
                                     delim_whitespace=True,
                                     error_bad_lines=False,
                                     skiprows=15).iloc[:, ind_mphi].dropna()
            colind = [12, 15, 3, 5, 4, 29, 31, 19, 21, 20]
            colstep = 38
            ind_mphi = [x + colstep * n_strainlim for x in colind]
            self.mPhi_long = pd.read_csv(aData.pSave + '\\' + str(aData.file_name) + '.txt',
                                         delim_whitespace=True,
                                         error_bad_lines=False,
                                         skiprows=15).iloc[:, ind_mphi].dropna()

        # Add column names
        if envelope:
            self.P_m = self.P_m.reindex(columns=['Px', 'Mx', 'Py', 'My'])

        self.P_m.columns = ['N (N)', 'Mxx (N-m)', 'N (N)', 'Myy (N-m)']
        self.M_phi.columns = ['Mxx (N-m)', 'phi_xx (1/m)', 'Myy (N-m)',
                              'phi_yy (1/m)']
        self.mPhi_long.columns = [
            'Mxx (N-m)',
            'phi_xx (1/m)',
            'ConfinedMinStrain_xx',
            'SteelMinStrain_xx',
            'SteelMaxStrain_xx',
            'Myy (N-m)',
            'phi_yy (1/m)',
            'ConfinedMinStrain_yy',
            'SteelMinStrain_yy',
            'SteelMaxStrain_yy']

        # Convert floating points to integers
        self.P_m = self.P_m.astype(int)

        # Initialize section area
        self.area = section_area

        # Read in plot settings
        if aData.devMode == 'on':
            plot_fig = True
        else:
            plot_fig = False

        # Get bi-linearized and cleaned M_phi curves
        if aData.cLimits == 'explicit':
            self.xP, self.yP, self.M_ult, self.k_ult, ind = util.expYield(
                self.mPhi_long, aData)
            self.ind_xx = ind[0]  # x-curvature index at Yield
            self.ind_yy = ind[1]  # y-curvature index at Yield
        else:
            if aData.cLimits == 'strict':
                constrained = True
            else:
                constrained = False
            self.xP, self.yP, M_phi_bilin, self.M_ult, self.k_ult = biLin(
                self.M_phi, aData.file_name, plot_fig=plot_fig, constrained=constrained, cleanCurves=aData.cleanCurves)
            self.ind_xx = np.searchsorted(np.squeeze(
                self.M_phi[['phi_xx (1/m)']]), self.xP[0])[0]  # x-curvature index at Yield
            self.ind_yy = np.searchsorted(np.squeeze(
                self.M_phi[['phi_yy (1/m)']]), self.xP[1])[0]  # y-curvature index at Yield

        # Write moment curvature to csv file
        self.M_phi.to_csv(aData.pSave + '\\' + 'Moment_Curvature_RAW_' +
                          aData.file_name + '.csv', index=False)

        # Clean moment curvature plots according to settings
        xM, xPhi = self.M_phi[self.M_phi.columns[0]
                              ], self.M_phi[self.M_phi.columns[1]]
        yM, yPhi = self.M_phi[self.M_phi.columns[2]
                              ], self.M_phi[self.M_phi.columns[3]]
        xM_raw, xPhi_raw = xM, xPhi
        yM_raw, yPhi_raw = yM, yPhi
        if aData.cleanCurves:
            xPhi, xM, numCutX = util.cutOutlier(xPhi, xM)
            yPhi, yM, numCutY = util.cutOutlier(yPhi, yM)
            self.ind_xx = self.ind_xx + numCutX
            self.ind_yy = self.ind_yy + numCutY

        # Plot interpretation of My if necessary
        if plot_fig:
            if aData.cLimits == 'explicit':
                plt.figure()
                plt.title('M_phi_xx-' + str(aData.file_name))
                plt.plot(xPhi, xM, 'o')
                plt.plot(xPhi_raw, xM_raw, marker='*')
                plt.plot(self.xP[0], self.yP[0], color='red', marker='^')
                plt.savefig(aData.pSave + '\\' + 'M_phi_xx-' +
                            str(aData.file_name) + '.png')
                plt.figure()
                plt.title('M_phi_yy-' + str(aData.file_name))
                plt.plot(yPhi, yM, 'o')
                plt.plot(yPhi_raw, yM_raw, marker='*')
                plt.plot(self.xP[1], self.yP[1], color='red', marker='^')
                plt.savefig(aData.pSave + '\\' + 'M_phi_yy-' +
                            str(aData.file_name) + '.png')

        # Implement an option for the user to specify whether they want to use
        # the fully bilinearized M-phi curve or just the point closest to the yield point from the
        # bilinearzed M-phi curve and XTRACT M-phi curve for all points
        # thereafter ("direct" method)
        if bilinearflag:
            print('         M-phi interpretation = bilinear')
            self.M_phi = M_phi_bilin
        else:
            print('         M-phi interpretation = direct (' + str(aData.cLimits) + ')')
            Mphi_xx = pd.DataFrame({'Mxx (N-m)': xM, 'phi_xx (1/m)': xPhi})
            Mphi_yy = pd.DataFrame({'Mxx (N-m)': yM, 'phi_xx (1/m)': yPhi})
            if len(Mphi_xx) < len(Mphi_yy):
                self.M_phi = pd.concat(
                    [Mphi_xx, Mphi_yy], axis=1, join_axes=[Mphi_xx.index])
            else:
                self.M_phi = pd.concat(
                    [Mphi_xx, Mphi_yy], axis=1, join_axes=[Mphi_yy.index])

        # Write curves in ".csv" files
        self.P_m.to_csv(aData.pSave + '\\' + 'Moment_Interaction_' +
                        aData.file_name + '.csv', index=False)
        self.M_phi.to_csv(aData.pSave + '\\' + 'Moment_Curvature_' +
                          aData.file_name + '.csv', index=False)

    def __process_curves__(self, aData):

        # Axial load (N)
        print('         Axial load = ' + str(aData.N_load) + 'kN')
        N = float(aData.N_load) * 1000

        # Shape of section
        shape = aData.section_type

        # Plastic hinge length (m)
        pl_hi_xx, pl_hi_yy = util.getPlasticHingeLength(aData)

        # Define moment of inertia
        if shape == 'Circular':
            # Diameter of circlular section (m)
            D = float(aData.diameter_circ) / 1000.0
            # Get the uncracked moment of inertia of the section (m4)
            I_xx = math.pi / 4 * (D / 2)**4
            I_yy = math.pi / 4 * (D / 2)**4

        elif (shape == 'Rectangular Beam') |\
             (shape == 'Rectangular Column'):
            # Get the width of the section (m)
            w = float(aData.rect_width) / 1000
            # Get the height of the section (m)
            h = float(aData.rect_height) / 1000
            # Get the uncracked moment of inertia of the section (m4)
            I_xx = w * h**3 / 12
            I_yy = h * w**3 / 12

        elif shape == 'T Beam':
            # Get the width of the flange (m)
            B = float(aData.f_width) / 1000
            # Get the height of the flange (m)
            h = float(aData.f_thick) / 1000
            # Get the width of the web (m)
            b = float(aData.t_width) / 1000
            # Get the height of the web (m)
            H = float(aData.t_height) / 1000 - float(aData.f_thick) / 1000
            # Get the center of gravity (m)
            A = B * h + H * b
            y_cog = ((H + h / 2) * h * B + H**2 * b / 2) / A
            # Get the uncracked moment of inertia of the section (m4)
            I_xx = b * H * (y_cog - H / 2)**2 + b * H**3 / 12 + \
                h * B * (H + h / 2 - y_cog)**2 + h**3 * B / 12
            I_yy = b**3 * H / 12 + B**3 * h / 12

        elif shape == 'Cruciform':
            # Total section width (m)
            bX = float(aData.bX) / 1000
            # Total section height (m)
            bY = float(aData.bY) / 1000
            # Horizontal flange width (m)
            tX = float(aData.tX) / 1000
            # Vertical flange width (m)
            tY = float(aData.tX) / 1000
            # Eccentricities (m)
            cX = float(aData.cX) / 1000
            cY = float(aData.cY) / 1000
            # Get the uncracked moment of inertia of the section (m4)
            I_xx = bX**3 * tX / 12 + (bY - tX)**3 * tY / 12
            I_yy = bY**3 * tY / 12 + (bX - tY)**3 * tX / 12

        # Define uncracked section properties
        self.I_noCrack_xx = I_xx
        self.I_noCrack_yy = I_yy

        # Define curves for each direction
        P_m_xx = self.P_m.iloc[:, :2]
        P_m_yy = self.P_m.iloc[:, 2:]

        P_m_xx = P_m_xx.sort_values(by=['Mxx (N-m)'], ascending=False)
        P_m_yy = P_m_yy.sort_values(by=['Myy (N-m)'], ascending=False)

        M_phi_xx = self.M_phi.iloc[:, :2]
        M_phi_yy = self.M_phi.iloc[:, 2:]

        # Calculate the moment plastic rotation curve
        def __M_theta_pl__(P_m, M_phi, E, pl_hi, N, sectionname, axis):

            # Slice the moment interaction diagram (full to half)
            P_m = P_m.iloc[:int(len(P_m) - (len(P_m) - 1) / 2), :]

            # Find compressive capacity based on give M-N diagram
            Ncom = max(P_m.iloc[:, 0])
            print('         Compressive capacity = ' + str(Ncom / 1000) + 'kN')

            # Find tensile capacity based on give M-N diagram
            Nten = min(P_m.iloc[:, 0])
            print('         Tensile capacity = ' + str(Nten / 1000) + 'kN')

            # Take appropriate outputs from bi-linearization
            if axis == 'xx':
                cl_ind = self.ind_xx
                phi_y = self.xP[0]
                My = self.yP[0]
                M_ult = self.M_ult[0]
                k_ult = self.k_ult[0]
                EI_noCrack = self.I_noCrack_xx * E
                I_noCrack = self.I_noCrack_xx
            else:
                cl_ind = self.ind_yy
                phi_y = self.xP[1]
                My = self.yP[1]
                M_ult = self.M_ult[1]
                k_ult = self.k_ult[1]
                EI_noCrack = self.I_noCrack_yy * E
                I_noCrack = self.I_noCrack_yy

            # Cracked section properties
            EI = My / phi_y
            I = EI / E
            ratio = I / I_noCrack

            # Get plastic curvature from the M-phi curve - φpl (1/m)
            print('         I_uncracked = ' + str(round(I_noCrack, 6)))
            print('         I_cracked = ' + str(round(I, 6)))
            print('         Ratio (I_cracked/_uncracked) = ' + str(round(ratio, 6)))
            print('         EI (uncracked) = ' + str(round(EI_noCrack, 0)))
            print('         EI (cracked) = ' + str(round(EI, 0)))
            print('         Yield curvature = ' + str(round(phi_y, 4)))

            # phi_plastic = phi - M/EI where EI = My/phi_y at yield point
            phi_pl = M_phi.values[:, 1] - M_phi.values[:, 0] / EI

            # Combine arrays with bending moment and plastic curvature
            M_phi_pl = np.vstack(
                (M_phi.values[:, 0], phi_pl)).transpose()[1:, :]

            # Drop plastic rotation rows with index < cl_ind
            M_phi_pl = M_phi_pl[cl_ind:, [0, 1]]

            # Drop plastic rotation rows with negative values (if they exist)
            M_phi_pl = M_phi_pl[M_phi_pl[:, 1] > 0]
            print('         My at specified N = ' + str(round(My, 0)))

            # Insert point where section yields
            M_phi_pl = np.vstack((np.array([My, 0]).reshape(1, 2), M_phi_pl))

            # Translate curve to start at phi_y = 0
            M_phi_pl[:, 1] = M_phi_pl[:, 1] - M_phi_pl[0, 1]

            # Remove repeated end values in M-Phi curve
            if M_phi_pl[len(M_phi_pl) - 1,
                        1] == M_phi_pl[len(M_phi_pl) - 2,
                                       1]:
                if aData.devMode == 'on':
                    print(
                        '         Removing repeated end values from moment curvature data...')
                M_phi_pl = M_phi_pl[:len(M_phi_pl) - 2, :]

            # Get plastic rotation - (radians)
            print('         Plastic hinge length = ' +
                  str(round(pl_hi, 5)) + 'm')
            theta_pl = M_phi_pl[:, 1] * pl_hi

            # Combine arrays with bending moment and plastic rotation
            M_theta_pl = np.vstack((M_phi_pl[:, 0], theta_pl)).transpose()

            # Convert arrays to dataframes
            M_phi_pl = pd.DataFrame(data=M_phi_pl,
                                    columns=['Moment (N-m)',
                                             'Plastic Curvature (1/m)'])
            M_theta_pl = pd.DataFrame(data=M_theta_pl,
                                      columns=['Moment (N-m)',
                                               'Plastic Rotation (radians)'])

            # Return moment - plastic rotation curve as well as other
            # properties
            return M_theta_pl, My, Ncom, Nten, I, phi_y, M_ult

        # Calculate moment - plastic rotation curve
        print('\n   Moment-plastic rotation (xx direction):')
        M_theta_pl_xx, My_xx, Ncom_xx, Nten_xx,\
            I_xx, phi_y_xx, M_ult_xx = __M_theta_pl__(P_m_xx, M_phi_xx, float(
                aData.Ec) * 1e6, pl_hi_xx, N, aData.file_name, 'xx')
        print('\n   Moment-plastic rotation (yy direction):')
        M_theta_pl_yy, My_yy, Ncom_yy, Nten_yy,\
            I_yy, phi_y_yy, M_ult_yy = __M_theta_pl__(P_m_yy, M_phi_yy, float(
                aData.Ec) * 1e6, pl_hi_yy, N, aData.file_name, 'yy')

        # Define the compressive and tensile axial capacity (N)
        Ncom = max(Ncom_xx, Ncom_yy)
        Nten = max(Nten_xx, Nten_yy)

        # Define the modulus of elasticity of concrete (Pa)
        E = float(aData.Ec) * 1e6

        # Concatenate curves
        M_theta_pl = pd.concat([M_theta_pl_xx, M_theta_pl_yy],
                               axis=1, join_axes=[M_theta_pl_xx.index])
        M_theta_pl.columns = ['Mxx (N-m)', 'ThetaP_xx (radians)',
                              'Myy (N-m)', 'ThetaP_yy (radians)']

        # Write curves in ".csv" files
        M_theta_pl.to_csv(aData.pSave + '\\' + 'Moment_Plastic_Rotation_' +
                          aData.file_name + '.csv', index=False)

        # Obtain shear capacity - axial load curve
        # Define a list with values of axial load
        incr = (Ncom - Nten) / 200
        axial_range = [incr * i for i in range(201)]
        axial_range = [Nten + i for i in axial_range]

        # Define the shear capacity in both directions for range
        VRdx, VRdy = [], []
        shear = Shear_Calcs(aData)
        for i in range(len(axial_range)):
            VRdx_temp, VRdy_temp = shear.Shear_cap(Ned=axial_range[i] / 1000)
            VRdx.append(VRdx_temp * 1000)
            VRdy.append(VRdy_temp * 1000)

        # Create arrays with shear - axial force values
        VRd_xx = np.array([axial_range, VRdx])
        VRd_yy = np.array([axial_range, VRdy])
        VRd_xx = pd.DataFrame(data=VRd_xx.transpose(),
                              columns=['N (N)', 'V_xx (N)'])
        VRd_yy = pd.DataFrame(data=VRd_yy.transpose(),
                              columns=['N (N)', 'V_yy (N)'])

        # Normalize arrays by axial force
        axial_range_n = [i / Nten if i <
                         0 else i / (-Ncom) for i in axial_range]
        VRd_xx_n, VRd_yy_n = VRd_xx, VRd_yy
        VRd_xx_n['N (N)'], VRd_yy_n['N (N)'] = axial_range_n, axial_range_n

        # Obtain shear force under expected axial load
        VRdx_ex, VRdy_ex = shear.Shear_cap(Ned=float(aData.N_load))
        VRdx_ex, VRdy_ex = VRdx_ex * 1000, VRdy_ex * 1000

        # Obtain shear force under zero axial load
        VRdx_noN, VRdy_noN = shear.Shear_cap(Ned=0)
        VRdx_noN, VRdy_noN = VRdx_noN * 1000, VRdy_noN * 1000

        # Return output arguments
        return M_theta_pl_xx, M_theta_pl_yy, My_xx, My_yy,\
            Ncom, Nten, E, VRd_xx, VRd_yy, VRd_xx_n, VRd_yy_n,\
            VRdx_ex, VRdy_ex, I_xx, I_yy, M_ult_xx, M_ult_yy,\
            VRdx_noN, VRdy_noN

    def write_mat(self, aData, PRS=[0, 0, 0, 0], PRT=[0, 0, 0, 0]):

        # Initialize curves
        M_theta_pl_xx, M_theta_pl_yy, My_xx, My_yy,\
            Ncom, Nten, E, VRd_xx, VRd_yy, VRd_xx_n,\
            VRd_yy_n, VRdx_ex, VRdy_ex, I_xx, I_yy, M_ult_xx,\
            M_ult_yy, VRdx_noN, VRdy_noN = self.__process_curves__(aData)
        print('\n   Final card data:')
        print('         Young\'s modulus = ' + str(round(E, 0)))
        print('         I_xx = ' + str(round(I_xx, 6)))
        print('         I_yy = ' + str(round(I_yy, 6)))
        print('         My_xx = ' + str(round(My_xx, 6)))
        print('         M_ult_xx = ' + str(round(M_ult_xx, 6)))
        print('         My_yy = ' + str(round(My_yy, 6)))
        print('         M_ult_yy = ' + str(round(M_ult_yy, 6)))
        print('         Section area = ' + str(round(self.area, 5)))
        print('         PRS = ' + str(PRS))
        print('         PRT = ' + str(PRT) + '\n')

        # Input parameters
        zero_10 = '0'.rjust(10)
        one_10 = '1'.rjust(10)

        # Determine area ratio (<1 if in-slab T-Beam)
        if aData.section_type == 'T-Beam' and aData.tbOption == 'in-slab':
            aEff = aData.section_area - aData.f_width * \
                aData.f_thick  # effective shear area
            f = aEff / aData.section_area  # factor to reduce density written to LS-DYNA
        else:
            f = 1

        # Parameters for first row of material card
        # Define the material id
        mid = str(aData.beam_num).rjust(10)
        # Define mass density of concrete (RO)
        ro = str(f * aData.den).rjust(10)
        # Define Young's modulus of concrete (E)
        e = "{:.1e}".format(E).replace('+', '').rjust(10)
        # Define Poisson's ratio of concrete (PR)
        pr = str(aData.v).rjust(10)
        # Abcissa definition for LCAT and LCAC
        iax = '2'.rjust(10)
        # Yield surface type of interaction
        isurf = '0'.rjust(10)
        # Isotropic hardening type
        ihard = '4'.rjust(10)
        # Flag for input of FEMA thresholds
        ifema = '3'.rjust(10)

        # Parameters for second row of material card
        # Yield moment vs rotation about local s-axis curve is (LCPMS)
        lcpms = str(10000 + (aData.beam_num) * 10).rjust(10)
        # Scale factor for yield moment force in LCPMS (SFS)
        sfs = str(int(M_ult_xx)).rjust(10)
        # Yield moment vs rotation about local t-axis curve is (LCPMT)
        lcpmt = str(10001 + (aData.beam_num) * 10).rjust(10)
        # Scale factor for yield moment force in LCPMT (SFT)
        sft = str(int(M_ult_yy)).rjust(10)
        # Axial tensile force vs strain curve id (LCAT)
        lcat = str(10002 + (aData.beam_num) * 10).rjust(10)
        # Scale factor for tensile force in LCAT (SFAT)
        sfat = str(abs(int(Nten))).rjust(10)
        # Axial compressive force vs strain curve id (LCAC)
        lcac = str(10003 + (aData.beam_num) * 10).rjust(10)
        # Scale factor for tensile force in LCAC (SFAC)
        sfac = str(abs(int(Ncom))).rjust(10)

        # Parameters for third row of material card
        # Moment interaction diagram for the local s-axis (ALPHA)
        pms = str(10004 + (aData.beam_num) * 10).rjust(10)
        pmsi = str(-10004 - (aData.beam_num) * 10).rjust(10)
        # Moment interaction diagram for the local t-axis (BETA)
        pmt = str(10005 + (aData.beam_num) * 10).rjust(10)
        pmti = str(-10005 - (aData.beam_num) * 10).rjust(10)
        # Pinching factor for flexural hysteresis (PINM)
        pinm = '0'.rjust(10)
        # Pinching factor for shear hysteresis (PINS)
        pins = '0'.rjust(10)
        # Location of plastic hinge 1 from node 1 (HLOC1)
        hloc1 = '0'.rjust(10)
        # Location of plastic hinge 2 from node 2 (HLOC2)
        hloc2 = '0'.rjust(10)

        # Parameters for fourth row of material card
        # Yield shear force vs inelastic shear strain (LCSHS)
        lcshs = str(10006 + (aData.beam_num) * 10).rjust(10)
        # Scale factor of yield shear force in the s-direction (SFSHS)
        sfshs = str(10007 + (aData.beam_num) * 10).rjust(10)
        sfshsi = str(-10007 - (aData.beam_num) * 10).rjust(10)
        # Yield shear force vs inelastic shear strain (LCSHS)
        lcsht = str(10008 + (aData.beam_num) * 10).rjust(10)
        # Scale factor of yield shear force in the t-direction (SFSHT)
        sfsht = str(10009 + (aData.beam_num) * 10).rjust(10)
        sfshti = str(-10009 - (aData.beam_num) * 10).rjust(10)
        # Define shear strength for expected axial force (N)
        vrdx = str(round(VRdx_ex, 0)).rjust(10)
        vrdy = str(round(VRdy_ex, 0)).rjust(10)
        # Define shear strength for 0 axial force
        vrdx0 = str(round(VRdx_noN, 0)).rjust(10)
        vrdy0 = str(round(VRdy_noN, 0)).rjust(10)

        # Parameters for eighth row of material card
        PRS = [str(round(i, 5)).rjust(10) for i in PRS]
        PRT = [str(round(i, 5)).rjust(10) for i in PRT]
        # Rotation limits in the local s-direction
        prs1 = PRS[0]
        prs2 = PRS[1]
        prs3 = PRS[2]
        prs4 = PRS[3]
        # Rotation limits in the local t-direction
        prt1 = PRT[0]
        prt2 = PRT[1]
        prt3 = PRT[2]
        prt4 = PRT[3]

        # Parameters for tenth row of material card
        # Shear strain thresholds in the local s-direction
        ss1 = zero_10
        ss2 = str(round(0.0002, 5)).rjust(10)
        ss3 = str(round(0.0005, 5)).rjust(10)
        ss4 = str(round(0.001, 5)).rjust(10)
        # Shear strain thresholds in the local t-direction
        st1 = zero_10
        st2 = str(round(0.0002, 5)).rjust(10)
        st3 = str(round(0.0005, 5)).rjust(10)
        st4 = str(round(0.001, 5)).rjust(10)

        # Define material card parameters
        fwrite = open(
            aData.pSave +
            '\\' +
            util.getName(
                aData,
                '.key',
                check=True),
            'w')
        fwrite.write('*KEYWORD\n')
        fwrite.write('$\n')
        fwrite.write('*MAT_HYSTERETIC_BEAM_TITLE\n')
        fwrite.write(util.getName(aData) + '\n')
        fwrite.write('%s%s%s%s%s%s%s%s\n' %
                     (mid, ro, e, pr, iax, isurf, ihard, ifema))
        fwrite.write('%s%s%s%s%s%s%s%s\n' %
                     (lcpms, sfs, lcpmt, sft, lcat, sfat, lcac, sfac))
        fwrite.write('%s%s%s%s%s%s%s%s\n' %
                     (pmsi, pmti, zero_10, zero_10, pinm, pins, hloc1, hloc2))
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (zero_10,
             zero_10,
             zero_10,
             zero_10,
             lcshs,
             sfshsi,
             lcsht,
             sfshti))
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10))
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10))
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10))
        fwrite.write('%s%s%s%s%s%s%s%s\n' %
                     (prs1, prs2, prs3, prs4, prt1, prt2, prt3, prt4))
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10))
        fwrite.write('%s%s%s%s%s%s%s%s\n' %
                     (ss1, ss2, ss3, ss4, st1, st2, st3, st4))
        fwrite.write('$\n')

        # Define section card parameters
        # Card 1
        sid = mid  # same as material id
        elform = '2'.rjust(10)
        shrf = '0.0'.rjust(10)
        qririd = '0.0'.rjust(10)
        cst = '0.0'.rjust(10)
        scoor = '0.0'.rjust(10)
        nsm = '0.0'.rjust(10)

        # Card 2
        area = str(float(self.area))
        iss = str(float(I_xx))
        itt = str(float(I_yy))
        area = ("{:.3E}".format(Decimal(area))).rjust(10)
        itt = ("{:.3E}".format(Decimal(itt))).rjust(10)
        iss = ("{:.3E}".format(Decimal(iss))).rjust(10)
        j = '0.0'.rjust(10)
        sa = '0.0'.rjust(10)
        ist = '0.0'.rjust(10)

        # Write section card
        fwrite.write('*SECTION_BEAM_TITLE\n')
        fwrite.write(util.getName(aData) + '\n')
        fwrite.write('%s%s%s%s%s%s%s\n' %
                     (sid, elform, shrf, qririd, cst, scoor, nsm))
        fwrite.write('%s%s%s%s%s%s\n' % (area, iss, itt, j, sa, ist))
        fwrite.write('$\n')

        # Define curves
        # Define plastic moment vs plastic rotation curve (s-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Yield Moment vs Rotation about local s-axis (LCPMS)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcpms,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        for i in range(0, len(M_theta_pl_xx)):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(M_theta_pl_xx.iloc[i, 1]))).rjust(
                20), str('%.4E' % Decimal(str(M_theta_pl_xx.iloc[i, 0] / M_ult_xx))).rjust(20)))
        fwrite.write('$\n')

        # Define plastic moment vs plastic rotation curve (t-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Yield Moment vs Rotation about local t-axis (LCPMT)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcpmt,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))

        for i in range(0, len(M_theta_pl_yy)):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(M_theta_pl_yy.iloc[i, 1]))).rjust(
                20), str('%.4E' % Decimal(str(M_theta_pl_yy.iloc[i, 0] / M_ult_yy))).rjust(20)))

        fwrite.write('$\n')

        # Define axial tensile force vs strain curve (s-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Axial Tensile Force vs Strain (LCAT)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcat,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        fwrite.write('                 0.0                 1.0\n')
        fwrite.write('                 0.1                 1.0\n')
        fwrite.write('$\n')

        # Define compressive tensile force vs strain curve (t-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Axial Compressive Force vs Strain (LCAC)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcac,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        fwrite.write('                 0.0                 1.0\n')
        fwrite.write('                 0.1                 1.0\n')
        fwrite.write('$\n')

        # Define moment interaction curve (s-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Moment Interaction curve about local s-axis (ALPHA)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (pms,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        # in DYNA, (+) Tension, (-) Compression.
        # in XTRACT, (+) Compression, (-) Tension
        # Multiply y axis of generated PM curve by -1
        for i in range(len(self.P_m)):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(self.P_m.iloc[i, 1]))).rjust(
                20), str('%.4E' % Decimal(str(self.P_m.iloc[i, 0] * -1.0))).rjust(20)))
        fwrite.write('$\n')

        # Define moment interaction curve (t-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Moment Interaction curve about local t-axis (BETA)\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (pmt,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        # in DYNA, (+) Tension, (-) Compression.
        # in XTRACT, (+) Compression, (-) Tension
        # Multiply y axis of generated PM curve by -1
        for i in range(len(self.P_m)):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(self.P_m.iloc[i, 3]))).rjust(
                20), str('%.4E' % Decimal(str(self.P_m.iloc[i, 2] * -1.0))).rjust(20)))
        fwrite.write('$\n')

        # Define yield shear vs inelastic shear strain curve (s-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write(
            'Yield Shear Force vs Inelastic Strain about local s-axis\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcshs,
             zero_10,
             zero_10,
             vrdx0,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        fwrite.write('                 0.0                 1.0\n')
        fwrite.write('                 0.1                 1.0\n')
        fwrite.write('$\n')

        # Define yield shear vs inelastic shear strain curve (t-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write(
            'Yield Shear Force vs Inelastic Strain about local t-axis\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (lcsht,
             zero_10,
             zero_10,
             vrdy0,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        fwrite.write('                 0.0                 1.0\n')
        fwrite.write('                 0.1                 1.0\n')
        fwrite.write('$\n')

        # Define shear vs axial force curve (s-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Shear Force vs Axial Force about local s-axis\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (sfshs,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        # The shear force is divided by the shear capacity VRdx_noN (vrdx0 string form)
        # Meaning unity at axial force = 0
        for i in reversed(range(len(VRd_xx_n))):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(VRd_xx_n.iloc[i, 0]))).rjust(
                20), str('%.4E' % Decimal(str(VRd_xx_n.iloc[i, 1] / VRdx_noN))).rjust(20)))
        fwrite.write('$\n')

        # Define shear vs axial force curve (t-axis)
        fwrite.write('*DEFINE_CURVE_TITLE\n')
        fwrite.write('Shear Force vs Axial Force about local t-axis\n')
        fwrite.write(
            '%s%s%s%s%s%s%s%s\n' %
            (sfsht,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             zero_10,
             one_10,
             zero_10))
        # The shear force is divided by the shear capacity VRdy_noN (vrdy0 string form)
        # Meaning unity at axial force = 0
        for i in reversed(range(len(VRd_yy_n))):
            fwrite.write('%s%s\n' % (str('%.4E' % Decimal(str(VRd_yy_n.iloc[i, 0]))).rjust(
                20), str('%.4E' % Decimal(str(VRd_yy_n.iloc[i, 1] / VRdy_noN))).rjust(20)))
        fwrite.write('$\n')
        fwrite.write('*END\n')

        # Close file handle
        fwrite.close()

        # Write output in JSON format
        keyFile = open(
            os.path.join(
                aData.pSave,
                util.getName(
                    aData,
                    '.key')),
            'r')
        keyData = keyFile.read()
        keyFile.close()
        jDat = {}
        jDat['file_name'] = util.getName(aData, '.key')
        jDat['value'] = keyData

        return jDat
